/* getblk.c, DCT block decoding                                             */

/* Copyright (C) 1994, MPEG Software Simulation Group. All Rights Reserved. */

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  The MPEG Software Simulation Group disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 * The MPEG Software Simulation Group does not represent or warrant that the
 * programs furnished hereunder are free of infringement of any third-party
 * patents.
 *
 * Commercial implementations of MPEG-1 and MPEG-2 video, including shareware,
 * are subject to royalty fees to patent holders.  Many of these patents are
 * general enough such that they are unavoidable regardless of implementation
 * design.
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "config.h"
#include "global.h"

/* defined in getvlc.h */
typedef struct {
  char run, level, len;
} DCTtab;

extern DCTtab DCTtabfirst[],DCTtabnext[],DCTtab0[],DCTtab1[];
extern DCTtab DCTtab2[],DCTtab3[],DCTtab4[],DCTtab5[],DCTtab6[];
extern DCTtab DCTtab0a[],DCTtab1a[];

DCTtab *DCTtable;

/* decode one intra coded MPEG-1 block */
int BuildDCTTable(void)
/*
inits the 64K DCT vlc table, returns true if successfull.
*/
{
	unsigned int code;
	DCTtab *tab;

	
	DCTtable = (DCTtab *)malloc(65536 * sizeof(DCTtab));
	if (DCTtable == NULL)
		return 0;

	//for each possible 16 code
	for(code = 0; code<65536; code++) {
		//look up value in "small" tables
	    if (code>=16384)
		  tab = &DCTtabnext[(code>>12)-4];
		else if (code>=1024)
		  tab = &DCTtab0[(code>>8)-4];
		else if (code>=512)
		  tab = &DCTtab1[(code>>6)-8];
		else if (code>=256)
		  tab = &DCTtab2[(code>>4)-16];
		else if (code>=128)
		  tab = &DCTtab3[(code>>3)-16];
		else if (code>=64)
		  tab = &DCTtab4[(code>>2)-16];
		else if (code>=32)
		  tab = &DCTtab5[(code>>1)-16];
		else  // (code>=16)
		  tab = &DCTtab6[code-16];
	
		DCTtable[code] = *tab;
		
		//process normal codes (not escape or end_of_block)
		if (tab->run < 64) {
			if (tab->len < 16) {
				//adjust value based on sign
				if (code & (0x00008000 >> tab->len))
					DCTtable[code].level = -tab->level;
				DCTtable[code].len++;  //don't forget to include sign bit in shift length
			}
			else {
				//for 16 bit codes, flag no sign info with run length
				DCTtable[code].len += 65;
			}
		}
	}

	return 1;
}

void FastGetIntraBlock(int comp, int dc_dct_pred[])
{
  int val, i, j;
  unsigned int code;
  DCTtab *tab;
  short *bp;

  bp = ld->block[comp];
  /* decode DC coefficients */
  if (comp<4)
    bp[0] = (dc_dct_pred[0]+=getDClum()) << 3;
  else if (comp==4)
    bp[0] = (dc_dct_pred[1]+=getDCchrom()) << 3;
  else
    bp[0] = (dc_dct_pred[2]+=getDCchrom()) << 3;

  if (fault) return;

  /* decode AC coefficients */
  for (i=1; ; i++)
  {
    code = showbits(16);
	tab = &DCTtable[code];

    if (tab->run>64) 
    {
		//escape case
		if (tab->run == 65) {
		  i+= (code>>4) & 0x3f;	   //adjust for run length of escape
		  flushbits(12);

		  val = getbits(8);
		  if(val>128)
			val -= 256;
		  else if ((val & 0x7f) == 0)	//if val = 0 or 128, read next 8 bits
			val = (val == 0) ? getbits(8) : getbits(8) - 256;
		}

		//16 bit code case
		else {
			i += tab->run - 65;
			flushbits(16);
			val = getbits(1) ? -tab->level : tab->level;
		}
    }
    else {
		//normal case & end_of_block case
		flushbits(tab->len);
        i+= tab->run;
        val = tab->level;
    }

    if (i > 63)
		return;

    j = zig_zag_scan[i];
    bp[j] = (val*ld->quant_scale*ld->intra_quantizer_matrix[j]) >> 3;

  }
}

