#if !defined(__FFT__)
#define __FFT__
#include <windows.h>

#include <math.h>
#define MINDB -1000.0F
#define PowerTodB(x) (((x) == 0.0F) ? MINDB : 10.0F * (float)log10(x))                

// error codes
typedef int ECODE;

#define OLNOERROR 0

#define OLDSP_BASE_ERROR 300
#define OLDSP_CANT_ALLOC_MEMORY               (OLDSP_BASE_ERROR + 0)
#define OLDSP_FFT_SIZE_TOO_LARGE              (OLDSP_BASE_ERROR + 1)
#define OLDSP_BAD_INPUT_DATA_WIDTH            (OLDSP_BASE_ERROR + 2)
#define OLDSP_BAD_FFT_SIZE                    (OLDSP_BASE_ERROR + 3)
#define OLDSP_NOT_ENOUGH_VALID_INPUT_DATA     (OLDSP_BASE_ERROR + 4)
#define OLDSP_WINDOW_SIZE_TOO_LARGE           (OLDSP_BASE_ERROR + 5)
#define OLDSP_BAD_DATA_TYPE                   (OLDSP_BASE_ERROR + 6)
#define OLDSP_BAD_WINDOW_TYPE                 (OLDSP_BASE_ERROR + 7)
#define OLDSP_NEGATIVE_INPUT_MAGNITUDE        (OLDSP_BASE_ERROR + 8)

typedef struct {
    float re;
    float im;
} COMPLEX;                  

typedef enum {
    UNSIGNED_DATA,
    SIGNED_DATA,
    FLOAT_DATA
} DATATYPE;
                                          
typedef enum {
    RECTANGLE,
    HANN,
    HANNING,
    HAMMING,
    BLACKMAN,
    BLACKMAN_HARRIS_67,
    BLACKMAN_HARRIS_92
} WINDOWTYPE;
                                          
//------------------ function prototypes-----------------------------
#ifdef __cplusplus
extern "C" {
#endif

ECODE InitFFT(void);  //can be used to pre-init FFT tables, not required
ECODE DoRealFFT(float *in, float *power, float *logmag, float *phase, int log2size);
void DoFFT(COMPLEX *data, int log2size);
void Unwind(COMPLEX *x, int ComplexFftSize);
ECODE BuildWindow(WINDOWTYPE WindowType,UINT WindowSize,float *CurrentWindow);
void FreeFFT(void);



#ifdef __cplusplus
}
#endif

#endif //!defined(__FFT__)
