/* tables.cpp
This file contains the intialization of the static tables in the CAudioEncode class.
These are taken from the MPEG 1 audio spec: ISO/IEC 11172-3: 1993.
*/

#include "encode.h"
#define NULL 0

const CAudioEncode::SQuantInfo CAudioEncode::QuantInfo[18] = {
    /*
    note that NormA and NormB are scaled so that qauntized bits are to the left
    of the binary point.  Also, NormB includes the offset needed to provide
    the required conversion to offset binary output.  This allows conversion
    before truncation so that the output will be truncated down for both
    positive and negative input
	*/

   //steps CodeSamps CodeBits TotalBits SNR     NormA                 NormB 
    {    0,  0,        0,      0,     0.00F, 0.000000000F*    1, (1.0F-0.000000000F)*    1}, //0
    {    3,  3,        5,     60,     7.00F, 0.750000000F*    2, (1.0F-0.250000000F)*    2}, //1
    {    5,  3,        7,     84,    11.00F, 0.625000000F*    4, (1.0F-0.375000000F)*    4}, //2
    {    7,  1,        3,    108,    16.00F, 0.875000000F*    4, (1.0F-0.125000000F)*    4}, //3
    {    9,  3,       10,    120,    20.84F, 0.562500000F*    8, (1.0F-0.437500000F)*    8}, //4
    {   15,  1,        4,    144,    25.28F, 0.937500000F*    8, (1.0F-0.062500000F)*    8}, //5
    {   31,  1,        5,    180,    31.59F, 0.968750000F*   16, (1.0F-0.031250000F)*   16}, //6
    {   63,  1,        6,    216,    37.75F, 0.984375000F*   32, (1.0F-0.015625000F)*   32}, //7
    {  127,  1,        7,    252,    43.84F, 0.992187500F*   64, (1.0F-0.007812500F)*   64}, //8
    {  255,  1,        8,    288,    49.89F, 0.996093750F*  128, (1.0F-0.003906250F)*  128}, //9
    {  511,  1,        9,    324,    55.93F, 0.998046875F*  256, (1.0F-0.001953125F)*  256}, //10
    { 1023,  1,       10,    360,    61.96F, 0.999023438F*  512, (1.0F-0.000976563F)*  512}, //11
    { 2047,  1,       11,    396,    67.98F, 0.999511719F* 1024, (1.0F-0.000488281F)* 1024}, //12
    { 4095,  1,       12,    432,    74.01F, 0.999755859F* 2048, (1.0F-0.000244141F)* 2048}, //13
    { 8191,  1,       13,    468,    80.03F, 0.999877930F* 4096, (1.0F-0.000122070F)* 4096}, //14
    {16383,  1,       14,    504,    86.05F, 0.999938965F* 8192, (1.0F-0.000061035F)* 8192}, //15
    {32767,  1,       15,    540,    92.01F, 0.999969482F*16384, (1.0F-0.000030518F)*16384}, //16
    {65535,  1,       16,    576,    98.01F, 0.999984741F*32768, (1.0F-0.000015259F)*32768}  //17
};                                                                               
                                                                                 
const CAudioEncode::SAllocationSequence CAudioEncode::AllocSequence[7] = {

    //table B.2a sb 0-2 and B.2b sb 0-2       							   0
    {4, 16,
        {
            &QuantInfo[ 0], &QuantInfo[ 1], &QuantInfo[ 3], &QuantInfo[ 5],
            &QuantInfo[ 6], &QuantInfo[ 7], &QuantInfo[ 8], &QuantInfo[ 9],
            &QuantInfo[10], &QuantInfo[11], &QuantInfo[12], &QuantInfo[13],
            &QuantInfo[14], &QuantInfo[15], &QuantInfo[16], &QuantInfo[17]
        }
    },
            
    //table B.2a sb 3-10 and B.2b sb 3-10        						   1
    {4, 16,
        {
            &QuantInfo[ 0], &QuantInfo[ 1], &QuantInfo[ 2], &QuantInfo[ 3],
            &QuantInfo[ 4], &QuantInfo[ 5], &QuantInfo[ 6], &QuantInfo[ 7],
            &QuantInfo[ 8], &QuantInfo[ 9], &QuantInfo[10], &QuantInfo[11],
            &QuantInfo[12], &QuantInfo[13], &QuantInfo[14], &QuantInfo[17]
        }
    },
            
	//table B.2a sb 11-22 and B.2b sb 11-22								   2
    {3, 8,																    
        {																    
            &QuantInfo[ 0], &QuantInfo[ 1], &QuantInfo[ 2], &QuantInfo[ 3], 
            &QuantInfo[ 4], &QuantInfo[ 5], &QuantInfo[ 6], &QuantInfo[17], 
            NULL,           NULL,           NULL,           NULL,           
            NULL,           NULL,           NULL,           NULL            
        }																    
    },																	    
    																	    
	//table B.2a sb 23-26 and B.2b sb 23-29								   3
    {2, 4,																    
        {																    
            &QuantInfo[ 0], &QuantInfo[ 1], &QuantInfo[ 2], &QuantInfo[17], 
            NULL,           NULL,           NULL,           NULL,           
            NULL,           NULL,           NULL,           NULL,           
            NULL,           NULL,           NULL,           NULL            
        }																    
    },																	    
    																	    
	//null entry, in the final subbands of each table B.2a - B.2d		   4
    {0, 1,																    
        {																    
            &QuantInfo[ 0], NULL,           NULL,           NULL,           
            NULL,           NULL,           NULL,           NULL,           
            NULL,           NULL,           NULL,           NULL,           
            NULL,           NULL,           NULL,           NULL            
        }																    
    },																	    
    																	    
    //table B.2c sb 0-1 and B.2d sb 0-1        							   5
    {4, 16,																    
        {																    
            &QuantInfo[ 0], &QuantInfo[ 1], &QuantInfo[ 2], &QuantInfo[ 4], 
            &QuantInfo[ 5], &QuantInfo[ 6], &QuantInfo[ 7], &QuantInfo[ 8], 
            &QuantInfo[ 9], &QuantInfo[10], &QuantInfo[11], &QuantInfo[12], 
            &QuantInfo[13], &QuantInfo[14], &QuantInfo[15], &QuantInfo[17]  
        }																    
    },																	    
    																	    
    //table B.2c sb 2-7 and B.2d sb 2-11        						   6
    {3, 8,
        {
            &QuantInfo[ 0], &QuantInfo[ 1], &QuantInfo[ 2], &QuantInfo[ 4],
            &QuantInfo[ 5], &QuantInfo[ 6], &QuantInfo[ 7], &QuantInfo[ 8],
            NULL,           NULL,           NULL,           NULL,          
            NULL,           NULL,           NULL,           NULL           
        }
    }
};

const CAudioEncode::SAllocationInfo CAudioEncode::AllocationInfo[4] = {
	//ISO/IEC 11172-3: 1993 table B.2a
	{
		&AllocSequence[0], &AllocSequence[0], &AllocSequence[0], &AllocSequence[1], 
		&AllocSequence[1], &AllocSequence[1], &AllocSequence[1], &AllocSequence[1], 
		&AllocSequence[1], &AllocSequence[1], &AllocSequence[1], &AllocSequence[2], 
		&AllocSequence[2], &AllocSequence[2], &AllocSequence[2], &AllocSequence[2], 
		&AllocSequence[2], &AllocSequence[2], &AllocSequence[2], &AllocSequence[2], 
		&AllocSequence[2], &AllocSequence[2], &AllocSequence[2], &AllocSequence[3], 
		&AllocSequence[3], &AllocSequence[3], &AllocSequence[3], &AllocSequence[4], 
		&AllocSequence[4], &AllocSequence[4], &AllocSequence[4], &AllocSequence[4]  
	},
	//ISO/IEC 11172-3: 1993 table B.2b
	{
		&AllocSequence[0], &AllocSequence[0], &AllocSequence[0], &AllocSequence[1], 
		&AllocSequence[1], &AllocSequence[1], &AllocSequence[1], &AllocSequence[1], 
		&AllocSequence[1], &AllocSequence[1], &AllocSequence[1], &AllocSequence[2], 
		&AllocSequence[2], &AllocSequence[2], &AllocSequence[2], &AllocSequence[2], 
		&AllocSequence[2], &AllocSequence[2], &AllocSequence[2], &AllocSequence[2], 
		&AllocSequence[2], &AllocSequence[2], &AllocSequence[2], &AllocSequence[3], 
		&AllocSequence[3], &AllocSequence[3], &AllocSequence[3], &AllocSequence[3], 
		&AllocSequence[3], &AllocSequence[3], &AllocSequence[4], &AllocSequence[4]  
	},
	//ISO/IEC 11172-3: 1993 table B.2c
	{
		&AllocSequence[5], &AllocSequence[5], &AllocSequence[6], &AllocSequence[6], 
		&AllocSequence[6], &AllocSequence[6], &AllocSequence[6], &AllocSequence[6], 
		&AllocSequence[4], &AllocSequence[4], &AllocSequence[4], &AllocSequence[4], 
		&AllocSequence[4], &AllocSequence[4], &AllocSequence[4], &AllocSequence[4], 
		&AllocSequence[4], &AllocSequence[4], &AllocSequence[4], &AllocSequence[4], 
		&AllocSequence[4], &AllocSequence[4], &AllocSequence[4], &AllocSequence[4], 
		&AllocSequence[4], &AllocSequence[4], &AllocSequence[4], &AllocSequence[4], 
		&AllocSequence[4], &AllocSequence[4], &AllocSequence[4], &AllocSequence[4]  
	},
	//ISO/IEC 11172-3: 1993 table B.2d
	{
		&AllocSequence[5], &AllocSequence[5], &AllocSequence[6], &AllocSequence[6], 
		&AllocSequence[6], &AllocSequence[6], &AllocSequence[6], &AllocSequence[6], 
		&AllocSequence[6], &AllocSequence[6], &AllocSequence[6], &AllocSequence[6], 
		&AllocSequence[4], &AllocSequence[4], &AllocSequence[4], &AllocSequence[4], 
		&AllocSequence[4], &AllocSequence[4], &AllocSequence[4], &AllocSequence[4], 
		&AllocSequence[4], &AllocSequence[4], &AllocSequence[4], &AllocSequence[4], 
		&AllocSequence[4], &AllocSequence[4], &AllocSequence[4], &AllocSequence[4], 
		&AllocSequence[4], &AllocSequence[4], &AllocSequence[4], &AllocSequence[4]  
	},
};

const CAudioEncode::SAllocationSetup CAudioEncode::AllocationSetup[11] = {

//rate/channel  44.1 KHz           48 KHz          32 KHz
    { -1,&AllocationInfo[1],&AllocationInfo[0],&AllocationInfo[1]},  //-1 =>free format
    { 32,&AllocationInfo[2],&AllocationInfo[3],&AllocationInfo[2]},
    { 48,&AllocationInfo[2],&AllocationInfo[3],&AllocationInfo[2]},
    { 56,&AllocationInfo[0],&AllocationInfo[0],&AllocationInfo[0]},
    { 64,&AllocationInfo[0],&AllocationInfo[0],&AllocationInfo[0]},
    { 80,&AllocationInfo[0],&AllocationInfo[0],&AllocationInfo[0]},
    { 96,&AllocationInfo[1],&AllocationInfo[0],&AllocationInfo[1]},
    {112,&AllocationInfo[1],&AllocationInfo[0],&AllocationInfo[1]},
    {128,&AllocationInfo[1],&AllocationInfo[0],&AllocationInfo[1]},
    {160,&AllocationInfo[1],&AllocationInfo[0],&AllocationInfo[1]},
    {192,&AllocationInfo[1],&AllocationInfo[0],&AllocationInfo[1]}
};


const CAudioEncode::SBitRates CAudioEncode::BitRates[15] = {
	{ -1,0xf}, //free data rate, all modes
	{ 32,0x8}, //mono only
	{ 48,0x8}, //mono only
	{ 56,0x8}, //mono only
	{ 64,0xf}, //all modes
	{ 80,0x8}, //mono only
	{ 96,0xf}, //all modes
	{112,0xf}, //all modes
	{128,0xf}, //all modes
	{160,0xf}, //all modes
	{192,0xf}, //all modes
	{224,0x7}, //all modes but mono
	{256,0x7}, //all modes but mono
	{320,0x7}, //all modes but mono
	{384,0x7}  //all modes but mono
};

const float CAudioEncode::ScaleFactorTable[SCALEFACTORTABLESIZE] = {
	2.00000000000000F, 1.58740105196820F, 1.25992104989487F,
	1.00000000000000F, 0.79370052598410F, 0.62996052494744F, 0.50000000000000F,
	0.39685026299205F, 0.31498026247372F, 0.25000000000000F, 0.19842513149602F,
	0.15749013123686F, 0.12500000000000F, 0.09921256574801F, 0.07874506561843F,
	0.06250000000000F, 0.04960628287401F, 0.03937253280921F, 0.03125000000000F,
	0.02480314143700F, 0.01968626640461F, 0.01562500000000F, 0.01240157071850F,
	0.00984313320230F, 0.00781250000000F, 0.00620078535925F, 0.00492156660115F,
	0.00390625000000F, 0.00310039267963F, 0.00246078330058F, 0.00195312500000F,
	0.00155019633981F, 0.00123039165029F, 0.00097656250000F, 0.00077509816991F,
	0.00061519582514F, 0.00048828125000F, 0.00038754908495F, 0.00030759791257F,
	0.00024414062500F, 0.00019377454248F, 0.00015379895629F, 0.00012207031250F,
	0.00009688727124F, 0.00007689947814F, 0.00006103515625F, 0.00004844363562F,
	0.00003844973907F, 0.00003051757813F, 0.00002422181781F, 0.00001922486954F,
	0.00001525878906F, 0.00001211090890F, 0.00000961243477F, 0.00000762939453F,
	0.00000605545445F, 0.00000480621738F, 0.00000381469727F, 0.00000302772723F,
	0.00000240310869F, 0.00000190734863F, 0.00000151386361F, 0.00000120155435F
};

float CAudioEncode::InvertedScaleFactors[SCALEFACTORTABLESIZE];

const CAudioEncode::SScaleFactorInfo CAudioEncode::ScaleFactorInfo[5][5] = {
//	sf0 sf1 sf2 SelInfo
  {
	{0,  1,  2,   0},
	{0,  1,  1,   3},
	{0,  1,  1,   3},
	{0,  2,  2,   3},
	{0,  1,  2,   0}
  },
  {
	{0,  0,  2,   1},
	{0,  0,  0,   2},
	{0,  0,  0,   2},
	{-1,-1, -1,   2},
	{0,  0,  2,   1}
  },
  {
	{0,  0,  0,   2},
	{0,  0,  0,   2},
	{0,  0,  0,   2},
	{2,  2,  2,   2},
	{0,  0,  2,   1}
  },
  {
	{1,  1,  1,   2},		
	{1,  1,  1,   2},		
	{1,  1,  1,   2},		
	{2,  2,  2,   2},
	{0,  1,  2,   0}
  },
  {
	{0,  1,  2,   0},
	{0,  1,  1,   3},
	{0,  1,  1,   3},
	{0,  2,  2,   3},
	{0,  1,  2,   0}
  }
};

const CAudioEncode::SScaleFactorSelInfo CAudioEncode::SFSelInfo[4] = {
// SFsent TotalBits 
   {3,       18},
   {2,       12},
   {1,        6},
   {2,       12}
};
