#ifndef __VIDEOCD__
#define __VIDEOCD__
/* VideoCD.h
   interface for video cd stream mixer
*/

#include "stream.h"
#include "..\errors.h"

#define VIDEO_CD_AUDIO_PACK_DATA_SIZE 2292  //#of data bytes for audio pack
#define VIDEO_CD_VIDEO_PACK_DATA_SIZE 2312  //#of data bytes for video pack
//#define VIDEO_CD_AUDIO_FILL_SIZE (VIDEO_CD_VIDEO_PACK_DATA_SIZE - VIDEO_CD_AUDIO_PACK_DATA_SIZE)
#define VIDEO_CD_AUDIO_FILL_SIZE 0
#define VIDEO_CD_PACK_SIZE (VIDEO_CD_VIDEO_PACK_DATA_SIZE + PACK_HEADER_SIZE) //total bytes of max pack
#define VIDEO_CD_AUDIO_PACKET_DATA_SIZE (VIDEO_CD_AUDIO_PACK_DATA_SIZE - AUDIO_PACKET_HEADER_SIZE)
//#define VIDEO_CD_VIDEO_PACKET_DATA_SIZE (VIDEO_CD_VIDEO_PACK_DATA_SIZE - VIDEO_PACKET_HEADER_SIZE)
#define VIDEO_CD_VIDEO_PACKET_MAX_DATA_SIZE (VIDEO_CD_VIDEO_PACK_DATA_SIZE - PACKET_HEADER_SIZE -2) // 1 for packet end flag and one stuffing byte
#define VIDEO_CD_PACK_TIME (1.0 / 75.0)
#define VIDEO_CD_AUDIO_FRAMES_AHEAD 2 //if there will be less then this number of
									 //audio frames in the decoders buffer after the next 
									 //video packet is send,  the next packet should be 
									 // audio data
#define VIDEO_CD_START_DELAY 30      //packet number of 1st data packet
#define VIDEO_CD_TARGET_VBV_FULLNESS 0 //(33*1024)
#define VIDEO_CD_INITIAL_VBV_FULLNESS (33*1024)
#define MAXDATAERROR (40*1024)

#define MIN_VIDEO_STUFFING 0  //(STD_BUFF_DATA_SIZE + END_CODE_SIZE+1) & 0xfe // 3 currently
//#define LESS_VIDEO_STUFFING (MIN_VIDEO_STUFFING+3)
//#define MORE_VIDEO_STUFFING (MAX_VIDEO_STUFFING-6)
#define MAX_VIDEO_STUFFING 16 + STD_BUFF_DATA_SIZE 
//min video data bytes = packet size - header size - 16 stuffing bytes
//#define MIN_VIDEO_CD_VIDEO_PACKET_DATA_SIZE (VIDEO_CD_VIDEO_PACK_DATA_SIZE - PACKET_HEADER_SIZE - MAX_VIDEO_STUFFING)


class CVideoCDMix : public CStreamMix {

    //interface functions
	public:
    CVideoCDMix(void) :  CStreamMix() { mLastVideoPacketDone=FALSE;mLastAudioPacketDone=FALSE;};
	virtual void ProcessData(void);
    virtual void FlushData(void);

    //helper functions
	private:

	virtual int GetVideoBufferSize(void) {return 46*1024;}   //get req size of decoder's buffer
    virtual int GetAudioBufferSize(void) {return 32*128; }   //get req size of decoder's buffer

	BOOL StartUpStream(void);
    void OutputSystemHeaders(void);
	void OutputVideoPack(BOOL DisableTimeStamp=FALSE);
	void OutputAudioPack(void);
	void OutputLastAudioPack(BOOL OutputLastFrameOnly=FALSE);
	void AdjustStreamTime(void);

	int NeedAudio(void);
	void OutputPackHeader(void);
	void IncrementPackCount() {
		mPackCount++;
		mAdjustedStreamTime += VIDEO_CD_PACK_TIME;
	}
	void OutputData(void const * data, int DataSize);
    void OutputZeros(int size);



	//constant data
	static const unsigned char AudioSystemHeader[15];
	static const unsigned char VideoSystemHeader[15];
	static const unsigned char EndCode[4];

	//object data
	private:
	BOOL mLastVideoPacketDone;   //False untill done
	BOOL mLastAudioPacketDone;	//False untill done
	double mStreamRate;			//total ouput rate in bytes/second
	long mPackCount;		    //running total of number of packs sent
	double mAdjustedStreamTime;	//stream time as adjusted for actual video rate
	double mAudioTimeAhead;		//(in seconds) if decoders buffers are less then this far
								// ahead, it is time to send more audio
    double mVBVPaddingTime;     //extra STD buffer padding to use (seconds)
    int mVideoStuffSize;        //total number of stuffing bytes to use on video packets
                                //that don't have time stamps

};

#endif //#ifndef __VIDEOCD__
