/* BasePin.cpp
This file implements the basic buffer, queue and BasePin classes.
CBasePin is a virtual base class.
*/

#include "basepin.h"

//------------------------------------------------------------------
//SBuff helper functions
//------------------------------------------------------------------

SBuff * AllocBuff(int size)
/*
This function allocates a buffer of the specified data size
*/
{
    SBuff *buff = (SBuff *)new char[sizeof(SBuff) - sizeof(buff->data) + size];
    buff->next = NULL;
    buff->DataSize = size;
    buff->ValidBytes = 0;
    return buff;
}

void FreeBuff(SBuff * buff)
/*
frees the specified buffer
*/
{
    delete [] (char *)buff;
}




//------------------------------------------------------------------
//CBuffQueue class
//------------------------------------------------------------------

CBuffQueue::CBuffQueue()
/*
constructor
*/
{
    mHead = mTail = NULL;
}

CBuffQueue::~CBuffQueue()
/*
distructor, free all queued buffers and critical section object
*/
{
    SBuff * buff;
    while ((buff = get()) != NULL)
        FreeBuff(buff);
}

SBuff *CBuffQueue::get(void)
/*
This funciton gets the next buffer from the head of the queue
*/
{
    mCritSec.Lock();
    SBuff *buff = mHead;
    if (mHead != NULL)
        mHead = mHead->next;
    mCritSec.Unlock();

    return buff;
}

void CBuffQueue::put(SBuff *buff)
/*
This funciton adds the specified buffer to the tail of the queue
*/
{
    mCritSec.Lock();
    buff->next = NULL;
    if (mHead == NULL) {
        mHead = mTail = buff;
    }
    else {
        mTail->next = buff;
        mTail = buff;
    }
    mCritSec.Unlock();
}


//------------------------------------------------------------------
//CBasePin virtual base class
//------------------------------------------------------------------

BOOL CBasePin::initialize(int NumBuffs, int BuffSize) 
/*
This function initializes the CBasePin object.  It should be called
once befor any other member function.  The function returns TRUE
if successfull, FALSE otherwise.
*/
{
    int i;
    SBuff * buff;

    //allocate and put buffers of empty queue
    for (i = 0; i<NumBuffs; i++) {
        buff = AllocBuff(BuffSize);
        if (buff == NULL) {
            //allocation error, free all buffers and exit with error
            while ((buff = mEmpty.get()) != NULL)
                FreeBuff(buff);
            return FALSE;
        }
        mEmpty.put(buff);
    }

    return TRUE;
}

SBuff *CBasePin::GetBuffer(int Timeoutms /*=0*/)
/*
fetch next empty buffer, NULL if no buffer available
*/
{
    return mEmpty.get();
}


void CBasePin::PutBuffer(SBuff * buff)
/*
put data buffer, and process input data
*/
{
    mFull.put(buff);
    ProcessData();
}
