#ifndef __BASEPIN__
#define __BASEPIN__

/* BasePin.h
This file defines the basic buffer, queue and BasePin classes.
CBasePin is a virtual base class.
*/

#include <windows.h>
#include "critsec.h"

struct SBuff {
    SBuff *next;    //linked list pointer
    int DataSize;   //total number of data bytes
    int ValidBytes; //current number of valid bytes
    char data[2];
};

SBuff * AllocBuff(int size);
void FreeBuff(SBuff * buff);


class CBuffQueue {
    SBuff *mHead;
    SBuff *mTail;
    CCritSec mCritSec;

    public:
    CBuffQueue();
    ~CBuffQueue();
    SBuff *get(void);
    void put(SBuff *buff);
    SBuff *peek(void) {return mHead;};
	BOOL IsReady() {return mHead!=NULL;};
};

class CBasePin {
    protected:
    CBuffQueue mEmpty;
    CBuffQueue mFull;

    public:
	BOOL BufferReady() {return mEmpty.IsReady();};
	BOOL initialize(int NumBuffs, int BuffSize);
    virtual SBuff *GetBuffer(int Timeoutms=0);
    virtual void PutBuffer(SBuff * buff);
    virtual void InputFinished(void) {};
    virtual BOOL IsInputFinished(void) {return FALSE;};

    protected:
    virtual void ProcessData(void) = 0;  //called after each put buffer
};


#endif //#ifndef __BASEPIN__