/*videopin.cpp
*/

#include "stream.h"
#include "..\errors.h"
#include <string.h>



BOOL CStreamMixVideoPin::initialize(int NumBuffs, int BuffSize)
/*
initialize reference queues and base class, return TRUE if ok
*/
{
	//test code
//    mTestFile =  CreateFile("test.mpg", GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,
//                                FILE_ATTRIBUTE_NORMAL, NULL);
	//end of test code
	
    int i;
    SBuff * buff;

    //allocate 1 frame reference structures for each data buffer
    //(more will get allocated if needed)
    //and put on unsed queue
    for (i = 0; i<NumBuffs; i++) {
        buff = AllocBuff(sizeof(SFrameRef));
        if (buff == NULL) {
            //allocation error, exit with error
            return FALSE;
        }
        mUnusedRefs.put(buff);
    }

	//set up 1st (dummy) frame reference for header data
	mInputFrameRefBuff = mUnusedRefs.get();
    SFrameRef *refptr = (SFrameRef *)mInputFrameRefBuff->data;
    refptr->FrameNumber = -1; //mark as header frame
    refptr->PresNumber = -1; //mark as header frame

    //init counters
    mFrameBytesInQueue = 0;
	mFrameBytesToXfer = 0;

	//init base class
    return CStreamMixAudioPin::initialize(NumBuffs, BuffSize);

}

void CStreamMixVideoPin::PutBuffer(SBuff * buff)
/*
process input video buffer
*/
{
	if (!mHeaderReady) {
		//if 1st buffer, read video header info
		ReadHeader(buff);
		mHeaderReady = TRUE;
	}

    int pos = 0;
    while (ParseVideo.FindNextPicture(buff, &pos)) {
        //fetch unused frame reference
		SBuff *RefBuff = GetEmptyFrameRef();
        if (RefBuff == NULL)
	        return;

        //update frame reverence info
        UpdateFrameRef(RefBuff);
    }

	//call base class 
    CStreamMixAudioPin::PutBuffer(buff);
}

SBuff *CStreamMixVideoPin::GetEmptyFrameRef(void)
/*
fetch unused frame reference
*/
{
    SBuff *RefBuff = mUnusedRefs.get();
    if (RefBuff == NULL) {
		//allocate new frame ref data buff, if all are in use
        RefBuff = AllocBuff(sizeof(SFrameRef));
        if (RefBuff == NULL)
            ProcessError(ERROR_MEMORY_ALLOCATION);
    }
	return RefBuff;
}


void CStreamMixVideoPin::UpdateFrameRef(SBuff *RefBuff)
/*
This function updates the queue of frame reference information.  It
uses the specifed buffer to store the current info on the new
video frame.  If the input reference buffer is NULL, the function
assumes that we are at the end of the stream and sets the sizes 
appropriatly.
*/
{
    //fill in frame data
    if (RefBuff != NULL) {
        SFrameRef *refptr = (SFrameRef *)RefBuff->data;
        refptr->FrameNumber = ParseVideo.GetFrameNumber();
        refptr->PresNumber = ParseVideo.GetPresNumber();
        refptr->VBVDelay = ParseVideo.GetVBVDelay();
    }

	//set frame size for last frame	(now that it is full)
	int FrameSize =	ParseVideo.GetLastFrameSize();
	if (RefBuff == NULL) {
		//adjust frame size for end of stream case
		//(include any extra data after the last frame)
		mCritSec.Lock();
		FrameSize = CStreamMixAudioPin::ValidData() - mFrameBytesInQueue;
		mCritSec.Unlock();
	}
	((SFrameRef *)(mInputFrameRefBuff->data))->FrameSize = FrameSize;

	//put last frame reference on mFrameRefs queue
    if (mCurFrameRefBuff == NULL) {
        //if no output reference buffer, load it now (start-up case)
		mCurFrameRefBuff = mInputFrameRefBuff;
		mCurFrameRef = (SFrameRef *)mCurFrameRefBuff->data;
		mFrameBytesToXfer = mCurFrameRef->FrameSize;
    }
    else
	    mFrameRefs.put(mInputFrameRefBuff);

    //update total bytes of complete frames in queue
	mCritSec.Lock();
	mFrameBytesInQueue += FrameSize;
	mCritSec.Unlock();

	//save the pointer to the new input frame info
	mInputFrameRefBuff = RefBuff;
}

int CStreamMixVideoPin::MoveData(char *OutPtr, int OutSize)
/*
This function moves the specified number of bytes of data
from the pin's buffers to the specified memory buffer.  The function
also updates the current frame information.	The function returns the
number of bytes actualy transfered.

*/
{
	//can't transfer more then we have
	int XferSize = MIN(OutSize, mFrameBytesInQueue);

	//call base class to move data
	XferSize = CStreamMixAudioPin::MoveData(OutPtr, XferSize);

	/*test code, write output to test file
    unsigned long TestSize;
    WriteFile(mTestFile, OutPtr, XferSize, &TestSize, NULL);
    if ((int)TestSize != XferSize)
        ProcessError(ERROR_FILE_WRITE);
	//end of test code */

	//update number of frame bytes in pin buffers
    mCritSec.Lock();
	mFrameBytesInQueue -= XferSize;
	mCritSec.Unlock();

	//remove any used frame ref structures
	int ToXfer = XferSize;
	while (ToXfer) {
		int size = MIN(mFrameBytesToXfer, ToXfer);
		ToXfer -= size;
		mFrameBytesToXfer -= size;
		//if end of frame, remove ref
		if (mFrameBytesToXfer == 0) {
			mUnusedRefs.put(mCurFrameRefBuff); //place ref buff on unused queue 
			mCurFrameRefBuff = mFrameRefs.get();
			if (mCurFrameRefBuff != NULL) {
				mCurFrameRef = (SFrameRef *)mCurFrameRefBuff->data;
				mFrameBytesToXfer = mCurFrameRef->FrameSize;
			}
			else {
				//handle end of stream case (no more frames)
				mCurFrameRef = NULL;
				mFrameBytesToXfer = 0;
			}
		}
	}

	return XferSize;
}

CStreamMixVideoPin::SFrameRef * CStreamMixVideoPin::GetNextFrameRef(void)
/*
returns a pointer to the reference structure of the next full frame
This routine should only be called if ValidData() >= RemainingFrameSize(void)
*/
{
	if (mFrameBytesToXfer != mCurFrameRef->FrameSize
                                || mCurFrameRef->FrameNumber == -1) {
        //ASSERT(mFrameRefs.peek() != NULL);
        SFrameRef *NextFrameRef = (SFrameRef *)(mFrameRefs.peek()->data);
		return NextFrameRef;
    }
	else {
		return mCurFrameRef;
    }
}



BOOL CStreamMixVideoPin::ReadHeader(SBuff * buff)
/*
This funciton reads information from the mpeg-1 sequence header that
is in the specified buffer.  If the sequence header is not found, or is
not complete, the function returns false.
*/
{
	struct SSeqHead {
		unsigned char stuff[3];		//stuff we don't need
        unsigned char FrameRate;    //aspect ratio-4 bits, frame rate code - 4 bits   
		unsigned char data[4];		//bit rate-18 bits, marker-1 bit, vbv size-10 bits, stuff -3 bits
	} *hdr;

	int SeqHeaderStartCode = 0x000001b3;
	int end = buff->ValidBytes - sizeof(SSeqHead);
	int acc = -1;
	int i;
	
	for (i=0; i<end; i++) {
		acc = (acc<<8) + (unsigned char)buff->data[i];
		if (acc == SeqHeaderStartCode) {
			hdr = (SSeqHead *)&buff->data[i+1];

            //convert frame rate
            static const double FrameRateTable[9] = {  //frame rate table form 11172-2 (frames/second)
                0.0, 23.976, 24.0, 25.0, 29.97, 30.0, 50.0, 59.94, 60.0
            }; 
            int FrameRateIndex = hdr->FrameRate & 0x0f;
            if (FrameRateIndex >= sizeof(FrameRateTable)/sizeof(FrameRateTable[0]))
                mFrameRate = 0.0;  //reserved value, set rate to zero
            else
                mFrameRate = FrameRateTable[FrameRateIndex];

			//convert bit rate info
			int BitRateCode = hdr->data[0];
			BitRateCode = (BitRateCode<<8) + hdr->data[1];
			BitRateCode = (BitRateCode<<2) + (hdr->data[2]>>6);
			if (BitRateCode == 0x3ffff)
				mDataRate = 0;	   //flag variable bit rate;
			else	
				mDataRate = (double)BitRateCode * (400/8);

            mAveFrameSize = mDataRate / mFrameRate;

			//convert VBV size
			int VbvCode = hdr->data[2] & 0x1f;
			VbvCode = (VbvCode<<5) + (hdr->data[3]>>3);
			mVBVSize = VbvCode * (16/8 * 1024);
			return TRUE;
		}
	}

	return FALSE;
}

double CStreamMixVideoPin::GetAveFrameSize(int length)
/*
This function returns the average data rate based on the number of complete frames
form the current frame back to the specified length.  The requested amount of data is
not available the function averages of all the data that is available.
*/
{
	int DataSize = 0;
	int frames = 0;
 	SBuff * RefBuff = mCurFrameRefBuff;

	while (RefBuff != NULL && DataSize < length) {
		int FrameSize = ((SFrameRef *)(RefBuff->data))->FrameSize;
		if (FrameSize == -1)
			break;
		DataSize += FrameSize;
		frames++;

		RefBuff = RefBuff->next;
	}
	
	if (frames)
		return 	DataSize/frames;
	else
		return 0.0;

}


void CStreamMixVideoPin::flush(void)
/*
This function is called when all input data has been put to the
object.  The function makes all data available to be output (via MoveData())
*/
{
    //update frame reference info
    UpdateFrameRef(NULL);

	//put last partial frame (if any) on queue by setting frame bytes to be
	//the total data bytes available to MoveData()
	mFrameBytesInQueue = CStreamMixAudioPin::ValidData();
}




//-------------------------------------------------------------------------------------------
//CParseVideo methods
//-------------------------------------------------------------------------------------------
const int CParseVideo::mPictureStartCode = 0x00000100;
const int CParseVideo::mGOPStartCode = 0x000001b8;

CParseVideo::CParseVideo(void)
/*
constructor for video stream parsing class
*/
{
	mAcch = mAccl = 0xffffffff;
    //init to size of saved stream, so 1st frame pos will be correct
	mBytesSinceLastFrame = 0 - sizeof(mAccl) -sizeof(mAcch);	
	mFrameNumber = -1;
	mPresNumber = -1;
	mLastFrameSize = -1;
	mMaxPresNumber = -1;
}


BOOL CParseVideo::FindNextPicture(SBuff * buff, int *position)
/*
This function searches the specified buffer starting at the specified position
for the next picture start code.  If the function finds a start code,
it updates the internal frame info, returns the current buffer position, and returns
TRUE.  If no picture start code is found, the function
returns false.

This function also searches for group of piture start codes to update the mGroupPresFrame
member variable.  mGroupPresFrame tracks the first presentation time (in cunits of frame count)
in the current group of pitures.
*/
#if 1
{
	//this version checks every byte, it also handles the case when start codes span
	//buffer boundries
    int end = buff->ValidBytes;
	int pos;

    for(pos = *position; pos < end; pos++) {
		//update stream data accumulator with next byte
		mAcch = (mAcch << 8) + (mAccl >> 24);
        mAccl = (mAccl << 8) + (unsigned char)buff->data[pos];
		mBytesSinceLastFrame++;
		  
		//compare to picture start code
        if (mAcch == mPictureStartCode) {
			//we found a new frame!!  Update frame info
			mFrameNumber++;
			mLastFrameSize = mBytesSinceLastFrame;
//			printf("Frame %d size %d\n",Frame++,mLastFrameSize);
            int PresOffset = mAccl >> (32 - 10);
			mPresNumber = PresOffset + mGroupPresFrame;
			while (mFrameNumber - mPresNumber > 512)
				mPresNumber += 1024;		//adjust for 10 bit modulo encoding if needed
			mMaxPresNumber = max(mMaxPresNumber, mPresNumber);
            unsigned int VBVDelayTicks =  (mAccl >> (32 - 29)) & 0x0000ffff;
            mVBVDelay = (double)VBVDelayTicks * (1.0/90000.0);  //convert to delay in seconds
//			printf("Frame %d, pres offset %d, pres # %d size %d\n",mFrameNumber,PresOffset,mPresNumber,mLastFrameSize); 
					 
            //update buff position and frame size
			mBytesSinceLastFrame = 0;
			*position = pos+1;
            return TRUE;
        }

		//compare to group of pictures start code
        else if (mAcch == mGOPStartCode) {
			//set 1st group pres time to be 1 more then the pres time of the last frame 
			mGroupPresFrame = mMaxPresNumber + 1;
//			printf("GOP: base = %d\n",mGroupPresFrame); 
		}

    }

    return FALSE;
}
#else
{
	//this faster version assumes start codes are on longword boundries
#define SEARCH_INC 4
    int end = buff->ValidBytes-SEARCH_INC;
	int pos;

    for(pos =*position; pos <=end; pos+=SEARCH_INC) {
		//update stream data accumulator with next dword
		mBytesSinceLastFrame+=SEARCH_INC;
		  
		//compare to picture start code
        if (mAcch == 0x00010000) {
			unsigned int mAccl = *(DWORD FAR*)(buff->data+pos); 
			DWORD tmp = mAccl;

			//swap bytes
			for (int x=0,y=3;x<4;x++,y--) {
				((char*)&mAccl)[x] = ((char*)&tmp)[y];
			}

			//we found a new frame!!  Update frame info
			mFrameNumber++;
			mLastFrameSize = mBytesSinceLastFrame;
            int PresOffset = mAccl >> (32 - 10);
			mPresNumber = PresOffset + mGroupPresFrame;
			while (mFrameNumber - mPresNumber > 512)
				mPresNumber += 1024;		//adjust for 10 bit modulo encoding if needed
			mMaxPresNumber = max(mMaxPresNumber, mPresNumber);
            unsigned int VBVDelayTicks =  (mAccl >> (32 - 29)) & 0x0000ffff;
            mVBVDelay = (double)VBVDelayTicks * (1.0/90000.0);  //convert to delay in seconds
//			printf("Frame %d, pres offset %d, pres # %d size %d\n",mFrameNumber,PresOffset,mPresNumber,mLastFrameSize); 
					 
            //update buff position and frame size
			mBytesSinceLastFrame = 0;
			*position=pos+SEARCH_INC;
	 		mAcch = *(DWORD FAR*)(buff->data+pos);
           return TRUE;
        }

		//compare to group of pictures start code
        else if (mAcch == 0xb8010000) {
			//set 1st group pres time to be 1 more then the pres time of the last frame 
			mGroupPresFrame = mMaxPresNumber + 1;
//			printf("GOP: base = %d\n",mGroupPresFrame); 
		}
		mAcch = *(DWORD FAR*)(buff->data+pos);

    }

    return FALSE;
}

#endif