/*video.cpp
This module contains the implementation of the CRecodeVideo class.  This class uses the
broadway hardware to re-compress the input data into IBP mpeg-1 video.  The compressed
data is written to the specified output file
*/

//
//  mpeg-1 video encode filter
//


//include our stuff
#include "video.h"
#include "errors.h"
#include "stream\basepin.h"

//standard lib includes
#include <memory.h>
#include <stdio.h>
#include <vcstruct.h>
#include <vcuser.h>
#include <vidtool.h>
#include <bwuser.h>
#include <bwdlgs.h>
//#include <audioenc.h>
#include "recode.h"
#include "audio\encode.h"
#include "stream\stream.h"
#include "stream\videoCD.h"
#include "resource.h"
#include "convaud.h"
#include <rcuser.h>
#include <math.h>

#define MAX_INPUT_DWORDS (0x20000 / sizeof(DWORD)) // 128K bytes
#define FOURCC_YUY2		mmioFOURCC('Y','U','Y','2')
#define FOURCC_UYVY		mmioFOURCC('U','Y','V','Y')
#define FOURCC_BROADWAY	mmioFOURCC('B','W','1','0')
#define FOURCC_BROADWAYlc	mmioFOURCC('b','w','1','0')
#define FOURCC_OLDBROADWAY	mmioFOURCC('B','M','P','G') )

#ifdef _DEBUG
static char* DebugMessage=__FILE__ ": ERROR %d returned from line %d\n";
static char DebugBuffer[120];
#define CHECK_ERROR(__func) {ECODE __ecode=__func; \
if (__ecode) {sprintf(DebugBuffer,DebugMessage,__ecode,__LINE__); \
OutputDebugString(DebugBuffer);return __ecode;} } 
#else
#define CHECK_ERROR(__func) {ECODE __ecode=__func; if (__ecode) return __ecode;} 
#endif


#define COMPRESS_TIME 2  //compres to IBP in 2 times real time
#define COMPRESS_BYPASS_TIME 3  //compres to IBP in 3 times real time
 

CRecodeVideo::CRecodeVideo(DEVID DevID) 
{
        mDevID = DevID;
		mTwoStepDecode=FALSE;  
		mBypass=FALSE;
		mpExtraBuffer=NULL;
		mInputFrameCount=-1;
		mSecondsRemaining=0;
		mCompressProgress=0;
		mbStreamBased=FALSE;
}

CRecodeVideo::~CRecodeVideo()
{
	delete [] mpExtraBuffer;
}


/* List of class IDs and creator functions for class factory */

//#define GREEN_FIX

ECODE CRecodeVideo::CompressVideoStream(PAVISTREAM pVideoStream, const char *OutFileName,	BOOL *AbortFlag,
                                                int DataRate/* = 1150000*/ , BOOL UseQSIF /*=FALSE*/,
												CBasePin *pVideoPin /* =NULL*/)
{
	mbStreamBased = TRUE;
	mVideoStream = pVideoStream;
	ECODE Status = CompressVideoStream((PAVIFILE)NULL
										,OutFileName
										,AbortFlag
										,DataRate
										,UseQSIF
										,pVideoPin
										);

	return Status;
}
												
ECODE CRecodeVideo::CompressVideoStream(PAVIFILE pAviFile, const char *OutFileName,	BOOL *AbortFlag,
                                                int DataRate/* = 1150000*/ , BOOL UseQSIF /*=FALSE*/,
												CBasePin *pVideoPin /* =NULL*/)
/*
This function uses the broadway hardware to compress the 1st video stream in the
specified AVI file.  The output mpeg-1 video stream is written to the specified file
*/
{
	mVideoPin=pVideoPin;
	//alloc data
	//CHECK_ERROR(VT_OpenDevice(TEXT("vtool"),0, &mDevID));
	//VT_AquireMutex();
	CHECK_ERROR(InitCompress(pAviFile, OutFileName, DataRate,UseQSIF));
	SetCompressProgress(0);


	//setup for progress calculations
    long TotalFrames = (long)(AVIStreamLength(mVideoStream)/mSrcFrameRate*mMpegFrameRate+0.5);
	double FrameCompressTime = COMPRESS_TIME / mMpegFrameRate;

	if (mBypass)
	{

		FrameCompressTime = COMPRESS_BYPASS_TIME / mMpegFrameRate;
	}
	else
	{
		mInData = mpExtraBuffer =new DWORD[MAX_INPUT_DWORDS];
	}

    //do the mpeg encode thing
    int InSize = MAX_INPUT_DWORDS * sizeof(DWORD); 
	ECODE status=0;
    //encode each frame in the avi stream
    while ( !*AbortFlag && mFrameCount<TotalFrames) {
        //printf("frame %3d bytes %d\n", start, BytesRead);  //test code
		status=GetFrame();
		if (status) 
		{
			*AbortFlag=TRUE;
			break;
		}
		SetFramesProcessed(mFrameCount+1);  // critical section not needed for reads


        //encode it & write to file!
        status=ProcessInput(mInData, mLastGoodFrameBytes/sizeof(DWORD));
		if (status) *AbortFlag=TRUE;

        //update compression progress
		SetCompressProgress(100.0 * mFrameCount / TotalFrames);
		SetSecondsRemaining((int)((TotalFrames - mFrameCount) * FrameCompressTime));
    }	

    if (!status)
		FinishProcessing();
	//VT_ReleaseMutex();

    if (mBypass)
	{
		AVIStreamGetFrameClose(mpGF);
	}
    //clean up
	if (!mbStreamBased)
		AVIStreamRelease(mVideoStream);

    CloseHandle(mOutFile);

	return status;


}

ECODE CRecodeVideo::GetFrame()
{
	int NextFrame=(int)(mFrameCount/mMpegFrameRate*mSrcFrameRate+0.5);
	long Size;
	if (NextFrame==mInputFrameCount)
	{
		return VT_NO_ERROR;
	}
	mInputFrameCount=NextFrame;
	if (mBypass)
	{
		mInData=(DWORD*) AVIStreamGetFrame(mpGF,mInputFrameCount);
		if (mInData==NULL) 
		{
			mLastGoodFrameBytes=0;
		}
		else 
		{
			mLastGoodFrameBytes=mBiOut.biSizeImage;
			mInData+=(sizeof(BITMAPINFOHEADER)/sizeof(DWORD));
			if (mTwoStepDecode) 
			{
				DWORD ec=ICCompress(mhIC, ICCOMPRESS_KEYFRAME,&mBiOut,mpExtraBuffer,&mBiIn,mInData,
							0,	
							NULL, // may not be legal	
							mFrameCount,	
							0,	
							0,	
							NULL,	
							NULL	
						   );	
				mInData=mpExtraBuffer;
			}
 
		}
	}
	else
	{
		long SamplesRead=0;
		AVIStreamRead(mVideoStream, mInputFrameCount, 1, mInData, MAX_INPUT_DWORDS * sizeof(DWORD), &Size, &SamplesRead); 
		//encode each frame in the avi stream
			//printf("frame %3d bytes %d\n", start, BytesRead);  //test code
		if (SamplesRead==0) 
		{
			mLastGoodFrameBytes=0;
		}
		else
		{
				//check for dropped frame
				//if this is a dropped frame, send last good frame (it's still in the buffer)
			if (Size != 0) {
				if ((mInData[0] & 0xff000000)==0xff000000)
				{
					if ((mInData[0] & 0x00ffffff) <= (DWORD)Size)
						Size=(mInData[0] & 0x00ffffff);
					mInData[0] = 0xb3010000;
					mInData[6]=0xf8ff0f00;  // fix up appairent bug in codec ucode

				}
				else
					if (mInData[0] != 0xb3010000)
						return VT_ERR_BAD_INPUT_VIDEO_TYPE;
				mLastGoodFrameBytes = Size;
			}
		}
	}
	return VT_NO_ERROR;
}

ECODE CRecodeVideo::InitCompress(PAVIFILE pAviFile, const char *OutFileName,
								 int DataRate, BOOL UseQSIF)
/*
This function set's up the memper data for the next compression.  It should be called 
before each compression.
*/
{
    //set hardware for video re-encoding
    // CHECK_ERROR( VT_Reset(mDevID));
   //CHECK_ERROR( VT_SetupVideoMode(mDevID,3,1));


    //create output file
    if (!mVideoPin)
	{
		mOutFile = CreateFile(OutFileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,
								FILE_ATTRIBUTE_NORMAL, NULL);
		if (mOutFile == NULL)
			return VT_ERR_CANT_OPEN_OUTPUT_FILE;
	}
	else 
		mOutFile = NULL;

	//open video stream
	if (!mbStreamBased) {
		if (0 != AVIFileGetStream(pAviFile, &mVideoStream, streamtypeVIDEO, 0)) {
		    CloseHandle(mOutFile);
			return VT_ERR_CANT_OPEN_VIDEO_STREAM;
		}
	}
    
    //check for I-frame input video
    AVISTREAMINFO StreamInfo;
    StreamInfo.fccHandler = 0;
    AVIStreamInfo(mVideoStream, &StreamInfo, sizeof(StreamInfo)); 
    if ( StreamInfo.fccHandler != mmioFOURCC('B','W','1','0')
		&&  StreamInfo.fccHandler != mmioFOURCC('b','w','1','0') // codec seems to give lower case
        && StreamInfo.fccHandler != mmioFOURCC('B','M','P','G') )
	{
		// init for bypass here 
		mBypass=TRUE;
        //CloseHandle(mOutFile);
     	//if (!mbStreamBased)
		   //AVIStreamRelease(mVideoStream);
        //return VT_ERR_BAD_INPUT_VIDEO_TYPE;
    }
#if 0
	//read frame rate form AVI file
	mSrcFrameRate = (double) StreamInfo.dwRate / StreamInfo.dwScale;
	//for now we will set to known NTSC or PAL rates
	if (mSrcFrameRate!=30.0)
		mMepgFrameRate = (mFrameRate >= 27.5) ? 29.97 : 25.00;
#endif
	//read frame size form AVI file
    long ReqSize;
    AVIStreamReadFormat(mVideoStream, 0, NULL, &ReqSize);
    BITMAPINFO *VideoFormat = (BITMAPINFO *)malloc(ReqSize);
    if (VideoFormat == NULL) {
        CloseHandle(mOutFile);
        if (!mbStreamBased)
			AVIStreamRelease(mVideoStream);
        return VT_ERR_CANT_OPEN_VIDEO_STREAM;
	}
    AVIStreamReadFormat(mVideoStream, 0, VideoFormat, &ReqSize);
	ECODE status=LoadRecoder(&VideoFormat->bmiHeader,(double)StreamInfo.dwRate / StreamInfo.dwScale);
	if (mBypass)
	{
		mpGF=AVIStreamGetFrameOpen(mVideoStream,&mBiOut);
		if (mpGF==NULL)
		{
			// try Rgb format with stretch
			mTwoStepDecode=TRUE;
			mBiIn.biCompression = BI_RGB;
			mBiIn.biWidth=mBiOut.biWidth;
			mBiIn.biHeight=mBiOut.biHeight;
			mBiIn.biSizeImage=0;

			mpGF=AVIStreamGetFrameOpen(mVideoStream,&mBiIn);
		}
		if (mpGF==NULL)
		{
			//try RGB no stretch default format 
			mBiIn.biWidth=VideoFormat->bmiHeader.biWidth;
			mBiIn.biHeight=VideoFormat->bmiHeader.biHeight;
			// should this be needed ????
			mBiIn.biSizeImage=0;
			//mBiIn.biSizeImage=VideoFormat->bmiHeader.biHeight*VideoFormat->bmiHeader.biWidth*2;
			mpGF=AVIStreamGetFrameOpen(mVideoStream,&mBiIn);
		}
		if (mpGF==NULL)
		{
			// PUNT
			// COULD TRY USING DEFAULT AND HOPE FOR THE BEST FROM PREP
			return VT_ERR_BAD_INPUT_VIDEO_TYPE;

		}
		if (mTwoStepDecode)
			status=PrepConverter();
	}
	
	free(VideoFormat);
	if (status)
	{
		// clean up we are returning an error
		CloseHandle(mOutFile);
		if (!mbStreamBased)
			 AVIStreamRelease(mVideoStream);
	}

    return status;
}

ECODE CRecodeVideo::PrepConverter()
{
	mhIC=ICOpen(ICTYPE_VIDEO,FOURCC_BROADWAY,ICMODE_COMPRESS);
	if (mhIC==NULL)
	{
		return VT_ERR_BAD_INPUT_VIDEO_TYPE;
	}
	DWORD ec=ICCompressBegin(mhIC,&mBiIn,&mBiOut);
	if (ec)
	{
		return VT_ERR_BAD_INPUT_VIDEO_TYPE;
	}
	mpExtraBuffer=new DWORD[mBiOut.biSizeImage/sizeof(DWORD)];
	return VT_NO_ERROR;
}

ECODE CRecodeVideo::ProcessInput(DWORD *InputData, int InputSize)
/*
This function compresses the specifed input video data, and writes the results to
the object's output file. (
*/
{
    //nothing to do if buffer empty
    if (InputSize==0)
        return 0;

    //read the current DMA address 
    DWORD addr;
	ECODE status=VT_GetCurrentTransferAddress(mDevID,2,&addr);
	if (status) return status;
    int loc = (addr-mDMAPhysAddr)/sizeof(CBT);

    //a bit of a kludge to chop off the 1st 16 DWORDS of zeros
    //so the Quartz mpeg player doesn't get messed-up, also
	//the 1st buffer must contain 1 complete video header
    if (mFirstWrite) {
        if (loc >= 1024) {
            mOutputBuffer.SetReadLocation(16);
            mFirstWrite = FALSE;
        }
    }
	else
    // remove the valid data from the DMA buffer
	{
   		mOutputBuffer.SetWriteLocation(loc);
		if (mVideoPin)
			mOutputBuffer.FlushToPin(mVideoPin); // do not flush 0 sized buffers
		else
			mOutputBuffer.FlushToFile(mOutFile);
	}

	// pad out the buffer
	DWORD size;
	if (InputSize<0x400)
		size=0x400-InputSize;
	else
		// INPUT size must be padded to blocks of 0x200 DWORDS
		size=(InputSize & 0xf) ? 0x10-(InputSize & 0xF):0;
	if (size)
	{
		ZeroMemory(&InputData[InputSize],size*sizeof(DWORD));
		InputSize+=size;
	}


    //send data to the board
    DWORD XferedCount;
	CHECK_ERROR(VT_WriteData(mDevID, InputData, InputSize, &XferedCount));
//	Sleep(10);
    //any remaining data to send, keep trucken 'tel it's gone
    size = InputSize-XferedCount;
	DWORD t1=timeGetTime();
    while (size )  {
        Sleep(5);  //don't use CPU while waiting
		CHECK_ERROR(VT_WriteData(mDevID, &InputData[XferedCount], size, &size));
        XferedCount += size;
        size = InputSize-XferedCount;
		if (timeGetTime()-t1 > 1000) 
			return VT_ERR_VIDEO_ENCODE_TIMEOUT;
        
    }
#if  0 //def _DEBUG
	char buffer[128];
	sprintf(buffer,"Processed %x size frame\n",InputSize);
	OutputDebugString(buffer);
#endif
	return 0;
}

#define STOP_CODE 0xb7010000

void CRecodeVideo::FinishProcessing()
/*
This function compresses any remaining data in the input buffer
*/
{
//    VT_Stop(mDevID);   //stop encoding, flush data from board  
	int count;
	int loops=0;
	DWORD temp;
	DWORD data[0x400];
    int size1,size2;
    CBT *ptr1, *ptr2;
    DWORD addr;
    ZeroMemory(data,sizeof(data));
	int size=mLastGoodFrameBytes/sizeof(DWORD);
	for (count=0;count<1;count++) ProcessInput(mInData,size); // send one extra last frame
	count=0;
	while (VT_WriteData(mDevID, NULL, 0, &temp) ==0 && (count++<5) ) 
		Sleep(20); // flush
	Sleep(500);         //wait for data to be output
	if (mBypass)
	{	
		VT_Stop(mDevID);
	}
	else
	{
		data[0]=0xb7010000; // stop code
		ProcessInput(data,0x400);
		while (VT_WriteData(mDevID, NULL, 0, &temp) ==0 ) ; // flush
	}
	//	while (!ProcessInput(pLastFrame,size) && count++<3) // allow extra frames for padding
//		while (VT_WriteData(mDevID, NULL, 0, &temp ==0 )) ;
	//Sleep(2000);         //wait for data to be output
	// wait for stop code ..
	BOOL found=FALSE;
	while (!found)
	{
		Sleep(500);
		VT_GetCurrentTransferAddress(mDevID, 2, &addr);
		mOutputBuffer.SetWriteLocation((addr-mDMAPhysAddr)/sizeof(CBT));
		// now scan the buffer
		mOutputBuffer.GetDataPointers(&ptr1, &size1, &ptr2, &size2);
		for (count=0;count<size1;count++)
		{
			if (*ptr1++==STOP_CODE)
			{
				mOutputBuffer.SetWriteLocation(ptr1);
				found=TRUE;
				break;
			}
		}
		for (count=0;count<size2;count++)
		{
			if (*ptr2++==STOP_CODE)
			{
				mOutputBuffer.SetWriteLocation(ptr2);
				found=TRUE;
				break;
			}
		}
		if (loops++>6 || found)
			break;

		if (mVideoPin)
			mOutputBuffer.FlushToPin(mVideoPin); // do not flush 0 sized buffers
		else
			mOutputBuffer.FlushToFile(mOutFile);

		ProcessInput(mInData,size); // send another frame
				
	}
#ifdef _DEBUG
	if (loops>6)
		OutputDebugStringA("End of stream not found\n");
#endif
	if (mVideoPin)
	{
		while (mOutputBuffer.ValidData())
		{
			mOutputBuffer.FlushToPin(mVideoPin);
		}
		mVideoPin->InputFinished();
	}
	else
		mOutputBuffer.FlushToFile(mOutFile);
	VT_Stop(mDevID);

	VT_SetMode(mDevID,VT_IDLE);
//	VT_Reset(mDevID);

}

#define rateerror(_x) fabs(_x/mSrcFrameRate-_x/(int)(mSrcFrameRate+0.5) )

ECODE CRecodeVideo::LoadRecoder(BITMAPINFOHEADER *Bi,double FrameRate)
{
	mSrcFrameRate=FrameRate;
	memcpy(&mBiIn,Bi,sizeof(mBiIn));
	memcpy(&mBiOut,Bi,sizeof(mBiIn));

	// fix up frame rate
	if (mSrcFrameRate!=30.0 && mSrcFrameRate >24.1)
		mMpegFrameRate = (mSrcFrameRate >= 27.5) ? 29.97 : 25.00;
	else if (mSrcFrameRate==24 || mSrcFrameRate==30)
		mMpegFrameRate=mSrcFrameRate;
	else
		if (rateerror(30)>rateerror(25))
			if (rateerror(24)>rateerror(25))
			   mMpegFrameRate=25;
			else
			   mMpegFrameRate=24;
		else
		   mMpegFrameRate=30;

    BW_RECODEOPTIONS Options;
	BW_RECODE_PREFERENCES Pref;
    /* before we do anything - try to get the recode settings */
	Options.ulStructSize=sizeof(Options);
    Options.nMode = MODE_FETCH_SETTINGS;
    Options.hWndParent = NULL;

	Pref.ulStructSize=sizeof(BW_RECODE_PREFERENCES);
	Options.lpPreferences=&Pref;

    if (!BW_GetRecodeOptions(&Options)) {
        return VT_ERR_BOARD_NOT_FOUND;
    }


	//read frame size form AVI file
	long FrameWidth = Bi->biWidth;
	long FrameHeight = Bi->biHeight;

	//download microcode to broadway hardware
#ifdef _DEBUG
	VT_Reset(mDevID); // remove before shipping
#endif
	RECODEINFO RecodeInfo;
	RecodeInfo.StructSize= sizeof(RECODEINFO);
	VT_GetRecodeInfo(mDevID,&RecodeInfo);
    RecodeInfo.FrameRate = (int)(mMpegFrameRate*100+0.5);
    RecodeInfo.DataRate  = Pref.nEncodeVideoBitRate;
	RecodeInfo.FrameWidth = (short)FrameWidth;
	RecodeInfo.FrameHeight = (short)FrameHeight;
	RecodeInfo.SequenceHeaderRepeatInterval=2;
    if ( Bi->biCompression != mmioFOURCC('B','W','1','0')
		&&  Bi->biCompression != mmioFOURCC('b','w','1','0') // codec seems to give lower case
        && Bi->biCompression != mmioFOURCC('B','M','P','G') )
	{
		// init for bypass here 
		mBypass=TRUE;
    }
	else 
		mBypass=FALSE;  // FOR NOW ALWASE BYPASS


	// determine frame info first base (input size)
	if (Pref.nEncodeSquarePixels)
	{
		RecodeInfo.FrameType=FT_SQPIXEL;
		if ((FrameWidth!=320) || (FrameHeight!=240))
		{
			mBypass=TRUE;
			RecodeInfo.FrameWidth=320;
			RecodeInfo.FrameHeight=240;
		}
	}
	else // Not fourcing to squair
	{
		if (FrameWidth==352 && FrameHeight==288)
			RecodeInfo.FrameType=FT_PAL;
		else if (FrameWidth==320 && FrameHeight==240 && (EStreamMixType)Options.nStreamMixType==StandardMix)
			RecodeInfo.FrameType=FT_SQPIXEL;
		else if (FrameWidth==352 && FrameHeight==240)
			RecodeInfo.FrameType=FT_NTSC;			
		else
		{
			mBypass=TRUE;
			double ratio=(double)FrameWidth/FrameHeight;
			if (ratio<=1.25)
			{
				RecodeInfo.FrameType=FT_PAL;
				RecodeInfo.FrameWidth=352;
				RecodeInfo.FrameHeight=288;
			}
			else if (ratio<=1.4 && (EStreamMixType)Options.nStreamMixType==StandardMix)
			{
				RecodeInfo.FrameType=FT_SQPIXEL;
				RecodeInfo.FrameWidth=320;
				RecodeInfo.FrameHeight=240;
			}
			else
			{
				RecodeInfo.FrameType=FT_NTSC;			
				RecodeInfo.FrameWidth=352;
				RecodeInfo.FrameHeight=240;
			}
		}
	}
	if (!Pref.nLargeImage)
	{
		// if using bypass sizes must be scaled
		//RecodeInfo.FrameWidth=(RecodeInfo.FrameWidth>>5) << 4;
		//RecodeInfo.FrameHeight=(RecodeInfo.FrameHeight>>5)<<4;
		switch (RecodeInfo.FrameType)
		{
			case FT_NTSC: RecodeInfo.FrameType=FT_NTSC_QSIF;

				break;
			case FT_PAL: RecodeInfo.FrameType=FT_PAL_QSIF;
				break;
			case FT_SQPIXEL: RecodeInfo.FrameType=FT_SQPIXEL_QSIF;
				break;
		}
	}


	VT_SetRecodeInfo(mDevID,&RecodeInfo);

	mBiOut.biHeight=-RecodeInfo.FrameHeight; // use inverted dib
	mBiOut.biWidth=RecodeInfo.FrameWidth;
	mBiOut.biCompression = FOURCC_UYVY;
	mBiOut.biSizeImage=RecodeInfo.FrameHeight*RecodeInfo.FrameWidth*2;
	mBiOut.biBitCount=16;

	//mBiIn.biHeight=RecodeInfo.FrameHeight;
	//mBiIn.biWidth=RecodeInfo.FrameWidth;
	//mBiIn.biCompression = ;
	//mBiIn.biSizeImage=RecodeInfo.FrameHeight*RecodeInfo.FrameWidth*2;
    
	ECODE status =  VT_SetMode(mDevID,VT_RECODE);
	if (status == VT_NO_ERROR && mBypass)
	{
		status = VT_SetParam(mDevID,RECODEBYPASS,TRUE);
		Sleep(1);
	}
//  to set sequenc header interval manualy
	VRP_COMMAND Cmd={8 /*EMI_LoadParams*/,5,(0x19000000L) /* TC_NOW */,5,5,2};
	VT_SendCommand(mDevID,&Cmd);
	
	if (status == VT_NO_ERROR)
		status = VT_Start(mDevID);
	if (status != VT_NO_ERROR) {
		//if error starting, clean-up and return
        CloseHandle(mOutFile);
        if (!mbStreamBased)
			AVIStreamRelease(mVideoStream);
        return status;
	}

	DWORD DMAWords;
	// These next two lines are a cludge to get the Buffer info from VT_access
	VT_ReadData(mDevID,&mDMALinearAddr, &DMAWords);
	VT_GetCurrentTransferAddress(mDevID,2,&mDMAPhysAddr);

    mOutputBuffer.Initialize((CBT*)mDMALinearAddr, DMAWords);
    //init state flags

	mLastGoodFrameBytes = 0;  //flag no good frames yet
    mFirstWrite = TRUE;
	mFrameCount = 0;
	return VT_NO_ERROR;
}
