#ifndef __CRECODEVIDEIO__
#define __CRECODEVIDEIO__
/* video.h
This module contains the interface of the CRecodeVideo class.  This class uses the
broadway hardware to re-compress the input data into IBP mpeg-1 video.  The compressed
data is written to the specified output file
*/

#define	WM_RECODE_STATUS_UPDATE (WM_USER + 177)

//system includes
#define WIN32_LEAN_AND_MEAN
#define DOSWIN32
#include <windows.h>
#include <vfw.h>

#include "vt_acc.h"
//#include <dt_scorp.h>
/*
extern "C" {
#include "vtfdioc.h"
}
*/

//includes for required classes
#include "cirbuf.h"
#include "stream\critsec.h"

class CBasePin;
class CRecodeVideo {

public:

    //constructor, load board and DMA buffer info
    CRecodeVideo(DEVID DevID) ;
	~CRecodeVideo();
    ECODE CompressVideoStream(PAVIFILE pAviFile, const char *OutFileName, BOOL *AbortFlag,
                                                int DataRate = 1150000, BOOL UseQSIF =FALSE,CBasePin *pVideoPin=NULL);

    ECODE CompressVideoStream(PAVISTREAM pVideoStream, const char *OutFileName, BOOL *AbortFlag,
                                                int DataRate = 1150000, BOOL UseQSIF =FALSE,CBasePin *pVideoPin=NULL);

	//thread safe compress progress access
#if 0
	int GetCompressProgress(void) {
		CAutoLock AutoLock(&mCritSec);
		return (int)mCompressProgress;
	}
#endif
	DOUBLE GetCompressProgress(void) {
		CAutoLock AutoLock(&mCritSec);
		return mCompressProgress;
	}
	void SetCompressProgress(double PercentProgress) {
		CAutoLock AutoLock(&mCritSec);
		mCompressProgress = PercentProgress;
	}
	int GetSecondsRemaining(void) {
		CAutoLock AutoLock(&mCritSec);
		return mSecondsRemaining;
	}
	void SetSecondsRemaining(int SecondsRemaining) {
		CAutoLock AutoLock(&mCritSec);
		mSecondsRemaining = SecondsRemaining;
	}

	int GetFramesProcessed(void) {
		CAutoLock AutoLock(&mCritSec);
		return mFrameCount;
	}

	void SetFramesProcessed(int FramesProcessed) {
		
		CAutoLock AutoLock(&mCritSec);
		mFrameCount=FramesProcessed;
	}
	
	ECODE ProcessFrame(BITMAPINFOHEADER *Bi,DWORD *InputData, int InputSize);

	ECODE LoadRecoder(BITMAPINFOHEADER *Bi,double FrameRate);
	ECODE PrepConverter();
    
private:

    /* Member variables */
    HANDLE mOutFile;            // output file handle
	CBasePin *mVideoPin; // output Video pin if null file is used
    CCircBuffer mOutputBuffer;  // circular buffer for board's output data
    BOOL mFirstWrite;           // true if haven't yet written to file
                                 // (used to skip 1st 16 DWORDs of data)
    PAVISTREAM mVideoStream;    // AVI video stream handle
	HIC	mhIC;
    int mLastGoodFrameBytes;    // size of the last good (non-dropped) frame
//    DWORD *mLastGoodFramePtr;    // the last good (non-dropped) frame
	int mInputFrameCount;			 // current number of frames read
	int mFrameCount;					// current frame output

    DEVID mDevID;               // handle to board
	PVOID mDMALinearAddr;            
	DWORD mDMAPhysAddr;
	DWORD *mInData;
	DWORD *mpExtraBuffer;
	BOOL	mBypass;			// input YUV data?
	BOOL    mTwoStepDecode;		// use BW10 codec to convert frame to YUV
	double mSrcFrameRate;			//input frame rate in frames/second
	double mMpegFrameRate;			//MPEG frame rate in frames/second
	double mCompressProgress;		//current compression progress in %
	int mSecondsRemaining;		//approx compression time remaining in seconds
    CCritSec mCritSec;			//critical section object
	BOOL mbStreamBased;         //TRUE if AVI stream based, FALSE if AVI file based


	BITMAPINFOHEADER mBiIn;		//Bitmap info header of input frames
	BITMAPINFOHEADER mBiOut;	//Bitmap info header of output (to board) frames
	//BOOL	mTranslationNeeded;  //input bitmap format must be translate to ouput
	PGETFRAME mpGF;				// Pointer to get frame object
    //helper functions
    ECODE InitCompress(PAVIFILE pAviFile, const char *OutFileName, int DataRate, 
						BOOL UseQSIF );
	//ECODE Init
	ECODE GetFrame();
    ECODE ProcessInput(DWORD *InputData, int InputSize);
    void FinishProcessing();


};

#endif //#ifndef __CRECODEVIDEIO__
