/**********************************************************************
 *
 * clm4100 loader - Application Programs Interface
 *
 * By:    William Chien
 * Date:  3 Jan 1994
 *
 * (C) Copyright 1994 - C-Cube Microsystems - All rights reserved
 *
 **********************************************************************/


/***********************************************************************
 *
 * CLM4100 Message Passing Shared Memory Addresses
 *
 ***********************************************************************/
//#include "4100_dmm.h"

#define MESSAGE_BASE	0x7F900

#define HOST_TO_CHIP_BUFFER	MESSAGE_BASE + 0X0
#define HOST_TO_CHIP_WRITE	MESSAGE_BASE + 0X100
#define HOST_TO_CHIP_READ	MESSAGE_BASE + 0X101
#define CHIP_TO_HOST_BUFFER	MESSAGE_BASE + 0X102
#define CHIP_TO_HOST_WRITE	MESSAGE_BASE + 0X202
#define CHIP_TO_HOST_READ	MESSAGE_BASE + 0X203
#define PRIORITY_BUFFER		MESSAGE_BASE + 0X204
#define TIMECODE		MESSAGE_BASE + 0X207
#define ALT_QMATRIX_FLAG	MESSAGE_BASE + 0X20B
#define INT_AREA                MESSAGE_BASE + 0x20D
#define INT_AREA_SIZE           8
#define POLARITY_BIT		MESSAGE_BASE + 0x215

/* download parameters */

#define	TRUE			1
#define	FALSE			0

#define END_OF_PARAMETERS       0
#define FILENAME_ROOT           1   /* char *   */
#define BITRATE                 2   /* double * */

#define POLARITY_FLAG           10
#define OPENLOOP_FLAG           11
#define DMEM_CLEAR_FLAG         12
#define PTS_REG_ADDR            13
#define AS_REG_ADDR             14
#define SCR_REG_ADDR            15
#define FREQ_FILT               20

#define PTS_DISABLED            0
#define PTS_FIXED_INCREMENT     0xffffffff
#define ASR_SCORPIO             0xf100001c
#define ASR_JUPITER             0xf1000020
#define SCR_SCORPIO             DA_DUMMY

#define TC_NOW                  (0x19000000L)
 

/*-----------------------------------------------------------------*/
/* API Message ID's and Parameter ID's
/*-----------------------------------------------------------------*/

#define SETOUTTIMECODE_MSG_ID      1
#define START_MSG_ID               3
#define STOP_MSG_ID                4
#define LOADPARAMS_MSG_ID          8
#define SETREFTIMECODE_MSG_ID      14

#define VIDEO_SOURCE_PARAM_ID   2
#define SEQUENCE_PARAM_ID       3
#define GOP_PARAM_ID            5
#define MATRIX_ID               7
#define MISCELLANEOUS_PARAM_ID  9


/*-----------------------------------------------------------------*/
/* Function Prototypes
/*-----------------------------------------------------------------*/

int clm4100_open(int chip);
int clm4100_close();
int clm4100_download(long *parameters);

/* fundamental message passing functions */
int clm4100_send_pri_msg(PREGS Regs, int type, int length, ulong *buf);
int clm4100_send_msg(PREGS Regs, int type, int length, ulong *buf);

/* priority message functions */
int clm4100_pSetRefTimeCode(ulong timecode);
int clm4100_pStart();
int clm4100_pStop();

/* standard message functions */
int clm4100_SetOutTimeCode(ulong sourceTC, ulong outputTC);
int clm4100_Start(ulong timecode);
int clm4100_Stop(ulong timecode);
int clm4100_LoadParams(int timecode, int paramID, int data_length, 
		       ulong *data);

