/* Copyright C-Cube Microsystems 1995	*/
/* @(#)RC_api.h	1.16 2/8/96 */
/**********************************************************************
 *
 * cl4000 loader - Application Programs Interface
 *
 * By:    William Chien
 * Date:  3 Jan 1994
 *
 * (C) Copyright 1994 - C-Cube Microsystems - All rights reserved
 *
 * V. Wilburn	20-Oct-94	Added PES support.
 * V. Wilburn	23-Jan-95	Added support for -NODEC, -WIDE_VIDA,
 *				and -WIDE_VIDB options.
 * V. Wilburn   08-Mar-95	Added support for -NOSCD and
 *				-NO_START_GOP_RESET options.
 * V. Wilburn	18-May-95	Added define for buffer length.
 * V. Wilburn	17-Jul-95	Added support -CMD_DISCARD_THRES and
 *				-BIT_DIST_CLIP_CTL options.
 * V. Wilburn	28-Jul-95	Added prototype for clm4110_IE_CloseGOP()
 *				and corrected typo in prototype for
 *				clm4110_IE_PicUserData().
 * V. Wilburn	15-Aug-95	Added new (and optional) rate control reset
 *				parameter to Start GOP command.
 * X. Magnisalis 14-Feb-96      Bit Stuffing removal changes
 * X. Magnisalis 14-Feb-96      added definitions for clm4500_Encode() 
 *                              and clm4500_Decode() neede in 4110 ICD
 *
 **********************************************************************/


/***********************************************************************
 *
 * CLM4500 Message Passing Shared Memory Addresses
 *
 ***********************************************************************/

#define MESSAGE_BASE	0x7F900

#define HOST_TO_CHIP_BUFFER	MESSAGE_BASE + 0X0
#define HOST_TO_CHIP_WRITE	MESSAGE_BASE + 0X100
#define HOST_TO_CHIP_READ	MESSAGE_BASE + 0X101
#define CHIP_TO_HOST_BUFFER	MESSAGE_BASE + 0X102
#define CHIP_TO_HOST_WRITE	MESSAGE_BASE + 0X202
#define CHIP_TO_HOST_READ	MESSAGE_BASE + 0X203
#define PRIORITY_BUFFER		MESSAGE_BASE + 0X204
#define TIMECODE		MESSAGE_BASE + 0X207
#define ALT_QMATRIX_FLAG	MESSAGE_BASE + 0X20B
#define INTERRUPT_ENABLE_BLOCK	MESSAGE_BASE + 0x20D
#define INTERRUPT_ENABLE_SIZE	0x8
#define POLARITY_BIT		MESSAGE_BASE + 0x215

#define HOST_TO_CHIP_BUFFER_LENGTH	(0x100)

/* download parameters */

#define	TRUE			1
#define	FALSE			0

#define END_OF_PARAMETERS       0
#define FILENAME_ROOT           1   /* char *   */
#define BITRATE                 2   /* double * */

#define POLARITY_FLAG           10
#define OPENLOOP_FLAG           11
#define DMEM_CLEAR_FLAG         12
#define PTS_REG_ADDR            13
#define AS_REG_ADDR             14
#define CL950_FLAG		15
#define INTERRUPT_ENABLE	16
#define PES_DATA		17
#define PTS_OFFSET		18
#define PTS_FORMAT		19
#define FREQ_FILT		20
#define DECIMATION_FLAG		21
#define WIDE_VIDA_FLAG		22
#define WIDE_VIDB_FLAG		23
#define SCD_FLAG		24
#define START_GOP_RESET_FLAG	25
#define CMD_DISCARD_THRES	26
#define BIT_DIST_CLIP_CTL	27
#ifdef CODEC
#define NO_STUFFING_FLAG        28  
#endif /* CODEC */
#define DECODER_FLAG            29

#define PTS_DISABLED            0
#define PTS_FIXED_INCREMENT     0xffffffff
#define PES_DISABLED		0xfffffffe
#define ASR_SCORPIO             0xf100001c
#define ASR_JUPITER             0xf1000020

#define TC_NOW			(0x19000000L)

/*-----------------------------------------------------------------*/
/* Function Prototypes
/*-----------------------------------------------------------------*/

int clm4110_IE_open(int ChipID);
int clm4110_IE_close();
int clm4110_IE_download(long *parameters);

/* fundamental message passing functions */
int clm4110_IE_send_pri_msg(int chip, int type, int length, ulong *buf);
int clm4110_IE_send_msg(int chip, int type, int length, ulong *buf);

/* priority message functions */
int clm4110_IE_pSetTimeCode(ulong timecode);
int clm4110_IE_pStart();
int clm4110_IE_pStop();
int clm4110_IE_pPause();
int clm4110_IE_pContinue();
int clm4110_IE_pReportStatus(int statusBlockID, ulong *status_data);

/* standard message functions */
int clm4110_IE_SetOutputTimeCode(ulong sourceTC, ulong outputTC);
int clm4110_IE_SetDefaultValues(ulong timecode, int defaultSetID);

int clm4500_Encode(ulong format);
int clm4500_Decode(ulong format);

int clm4110_IE_Pause(ulong timecode);
int clm4110_IE_Continue(ulong timecode);
int clm4110_IE_LoadParams(int timecode, int paramID, int data_length, 
		       ulong *data);
int clm4110_IE_ReportStatus(ulong timecode, int statusBlockID, 
			 ulong *status_data);
int clm4110_IE_StartGOP(ulong timecode, ulong seqhdr, ulong rc_reset);
int clm4110_IE_SeqUserData(ulong *data, ulong length);
int clm4110_IE_GOPUserData(ulong *data, ulong length);
int clm4110_IE_PicUserData(ulong *data, ulong length);
int clm4110_IE_CloseGOP(ulong timecodeUL, ulong argUL);

/* auxillary functions */
void clm4110_IE_loadClippingTables(int horizontal_mb_resolution, 
				int vertical_mb_resolution);

