/*MakeKey.cpp

KEEP THIS CODE SAFE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


This file contains the code and definitions to generate the
Broadway software key. 


The MakeKey() function produces a 64 bit key from a 60 bit input number.
This input should be a linear encode (i.e. bit munge) of a boards
serial number and rights data. The CheckKey() functon (see checkKey.cpp)
can be used to decode the key.

This code uses privat/public key encryption (the RSA algorithm) to generate the key.
This file contains the private definitions SO KEEP IT SAFE. 

This module also contains some helper functions that were used to help compute
the key definitons.
*/


#include "KeyFuncs.h"

#define PRIVATE_KEY_D 1271473011
#define PRIVATE_KEY_P1 65519UL
#define PRIVATE_KEY_P2 65327UL
#define PUBLIC_KEY_n (PRIVATE_KEY_P1 * PRIVATE_KEY_P2)

unsigned long MakeKey32(unsigned long input30)
{
    return ExpMod(input30 & 0x3fffffff, PRIVATE_KEY_D, PUBLIC_KEY_n);
}


UINT64 MakeKey(UINT64 PlainText)
/*
This function encodes a 64 bit key from the input 60 bit plain text
*/
{

    unsigned long inl, inh, outl, outh;
    UINT64 result;

    //encode the least sig 30 bits of plain text
    inl = (unsigned long) PlainText;
    inl &= 0x3fffffff;
    outl = ExpMod(inl, PRIVATE_KEY_D, PUBLIC_KEY_n);
     
    //encode next 30 bits of plain text
    inh = (unsigned long) (PlainText >> 30);
    inh &= 0x3fffffff;
    outh = ExpMod(inh, PRIVATE_KEY_D, PUBLIC_KEY_n);

    //combine LSH and MSH into 64 bit key
    result = outh;
    result <<= 32;
    result += outl;

    return result;
}



//helper functions that were used the calc the key definitions
#include <math.h>

int IsPrime(unsigned int num)
/*
this brute force function returns true if the input is a prime number
*/
{
    int max = (int)sqrt((double)num);

    for (int i=2;i<max;i++) {
        if (num % i == 0)
            return 0;
    }

    return 1;
}

unsigned long ComputeDecodeExp(unsigned long P1, unsigned long P2, unsigned long E)
/*
computes the decode exponient when given the two private key prime numbers and
the encode (public key) exponient.

we must compute DecodeExp so that:

E*DecodeExp mod ((P1-1)*(P2-1) = 1

note that we could use the Euclidean algorithm here, but we are going brute force.
This code just tries values of DecodeExp till the above is true. However, instead
of trying all, we increament DecodeExp so that the remander is > base - E or < E.
*/
{

    unsigned base = (P1-1) * (P2-1);
    unsigned long q = base / E;
    __int64 r = 0;                //init remander
    unsigned long DecodeExp = 0;

    do {
        unsigned long next = q;
        if (r > E)
            next++;
        r = (r + next * E) % base;
        DecodeExp += next;
    } while (r != 1); 
        

    return DecodeExp;
}