#include "stdincs.h"
#include <stdlib.h>
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include "dtdefs.h"
#include "stdio.h"
#include "vt_acc.h"
#include "vt_regs.h"
#include "vt_vrp.h"
#include "4000_reg.h"
#include "capture_dmm.h"
#include "4100_ld.h"
#include "4100_api.h"
#include "mcode.h"
#include "capture.h"
#include "LbrCapture.h"
#include "math.h"

CLbrCapture::CLbrCapture(DEVSTRUCT *dwDevice) :CCapture(dwDevice) 
{
	m_DA_CLIPX=DA_CLIPX;
	m_DA_CLIPY=DA_CLIPY;
	m_DA_PTB=DA_PTB;
	m_DA_BTB=DA_BTB;
	m_ME1_CLIP_TABLE_P=ME1_CLIP_TABLE_P;
	m_ME1_CLIP_TABLE_B=ME1_CLIP_TABLE_B;
	PhysBuffer=NULL;
	UcodeFileName="lbr_capt.dmb";
}

static int pts_reg_value   = PTS_DISABLED;
static int scr_reg_value   = PTS_DISABLED;
static int as_reg_value    = DA_DUMMY;
static int int_area_value  = 0;
static int freq_filt_enable = 0;

ECODE CLbrCapture::Load()
{
	PREGS Regs=GET_STRUCT_PTR(Dev);
	if (!(Dev->GetBoardCaps() & BROADWAY_CANCAPTURE_MPEG))
	{
		Dev->ShowMessage(VT_ERR_HARDWARE_NOT_CAPABLE);
		return VT_ERR_HARDWARE_NOT_CAPABLE;
	}

	if (!(Dev->GetBoardCaps() & BROADWAY_CANDO_VCD))
	{
		int bitrate=Dev->CaptureInfo.DataRate;
		if (bitrate > 1100000 && bitrate <1160000)
		{
			Dev->ShowMessage(VT_ERR_HARDWARE_NOT_CAPABLE);
			return VT_ERR_HARDWARE_NOT_CAPABLE;
		}
	}

 // PREGS Regs;
  uwide temp36;
  VT_ResetVRP(Dev); 
  VT_ResetVRP(Dev); 
  

    
    /* reset chip & clear IMEM - set up for initialization sequence */
    
    reset(Regs);
    imemClear(Regs);
    
    imemWrite(Regs,0x10,0x8c0);
    run_inst(Regs,32);
    imemWrite(Regs,0x10,0);
    reset(Regs);
    //smemClear(Regs);
    
    /*  Enable DMA Unit (Normal, Interleaved, Refr Count = 0x100). */
    GBusWrite(GBUS_DMAMODE,VIDTOOL_MEMCONFIG); 
    
    /*  Reset Vlc_Start register. */
#define SC_4010  //??????
#ifdef SC_4010	/* E2: Use Autostart for ZZ xfer */
    /* Reset Vlc_Start register and set AutoStart feature ON */
    GBusWrite(GBUS_VLCSTART,0x4000);
#else   
    /* Reset Vlc_Start register. */
    GBusWrite(GBUS_VLCSTART,0x0);
#endif
    /*  Reset then turn On VLC Unit (MPEG, Intra, empty PACK & ZZ). */
    GBusWrite(GBUS_VLCCTL,0x00080105);
    GBusWrite(GBUS_VLCCTL,0xc0080105);

    /*  Enable CPU module (reset Ghalt, Greset), set PIO as input. */
    WBusWrite(Regs,WBUS_CPUCTL,0xc0000000);
    
    /*  Dummy VLD transfer to disable VLD. */
    WBusWrite(Regs,WBUS_CHMEM5,0x07840002);
    
    /*
     * set a breakpoint
     * run for awhile to initialize the VLD
     * then reset the chip. 
     */
    GBusWrite(0x24,0x20);
    GBusWrite(0x24,0x0);
    
    GBusWrite(GBUS_VLDDATA,0x0);
    GBusWrite(GBUS_VLDDATA,0x0);
    GBusWrite(GBUS_VLDDATA,0x0);
    GBusWrite(GBUS_VLDDATA,0x0);

    GBusWrite(GBUS_VLDDATA,0x0);
    GBusWrite(GBUS_VLDDATA,0x0);
    GBusWrite(GBUS_VLDDATA,0x0);
    GBusWrite(GBUS_VLDDATA,0x0);
    
    GBusWrite(GBUS_VLDDATA,0x0);
    GBusWrite(GBUS_VLDDATA,0x0);
    GBusWrite(GBUS_VLDDATA,0x0);
    GBusWrite(GBUS_VLDDATA,0x0);
    
    GBusWrite(GBUS_VLDDATA,0x0);
    GBusWrite(GBUS_VLDDATA,0x0);
    GBusWrite(GBUS_VLDDATA,0x0);
    GBusWrite(GBUS_VLDDATA,0x0);
    

    /*  VLD initialized: start again. */
    reset(Regs);                             

    rmemClear(Regs);
#if 0
    //if(dmem_clr == TRUE) {
      dramClear(Regs);
    //}
#endif   
	// Init board here ?	
    
    /* initialize rmem locations - MUST BE DONE BEFORE INITIALIZING IMEM! */
    
    temp36.loword = 0xc00000;
    temp36.hiword = 0;
    rmemWrite(Regs,22,&temp36);
    CpuWritePC(Regs, 0, 0, 0);

    temp36.loword = 0x800000;
    temp36.hiword = 0;
    rmemWrite(Regs,23,&temp36);
    CpuWritePC(Regs, 0, 0, 0);

    temp36.loword = 0;
    temp36.hiword = 0;
    rmemWrite(Regs,30,&temp36);
    CpuWritePC(Regs, 0, 0, 0);
    
    /* Initialize initial DMEM image */
    //VT_LoadFile(Dev,VR_DRAM,"capture.dmb");
    /* printf("loading %s in dram # %d\n",dmem_file,chip); */
    
    /* Initialize initial IMEM image */
    //VT_LoadFile(Dev,VR_IMEM,"capture.imb");
     ReturnIfError(LoadMicrocode(UcodeFileName));
/* printf("loading %s in imem # %d\n",imem_file,Regs); */
    
	// setup line blanking
	dramWrite(Regs,DA_VCR_BLANK>>2,Dev->CaptureInfo.Blanklines);


    /* Load the Tables and do the remainder of the initialization.  If SIF 
       resolution, make sure to initialize the rc and bitrate parameters! */
	
    

    init_SIF();   

    /*  set the chipid */    
//    GBusWrite(chip,GBUS_HCTL,0x50001050);
    GBusWrite(GBUS_HCTL,0x50001000 | (CHIP_ID << 4));
    
    /* start the chip executing */
    
	GBusWrite(GBUS_HCTL,0xf0000000);
	Sleep(10);
//	return  VT_NO_ERROR;
  // if pal inputs 
	DWORD data[6];
    if (Dev->CurrentVideoMode == PAL) 
	{
		/*---------------------------------------------------------
		 *  Set the encoder mode to PAL
		 *---------------------------------------------------------*/
		data[0] = 6;  /* Video Input Mode */
		data[1] = 7;  /* PAL 710x576 input */ 

		ReturnIfError(LoadParams(TC_NOW, 2, 2, data));

	}

	ReturnIfError( SetDefaultValues(TC_NOW, Dev->CaptureInfo.FrameType));

	data[0] = 1;  /* Bit rate */
	data[1] = Dev->CaptureInfo.DataRate/400;;  /* new bitrate*/ 
	ReturnIfError(LoadParams(TC_NOW, 3, 2, data));

	// if !IBP set Gop size 1
	if (!Dev->CaptureInfo.IBP)
	{
		data[0]=1;
		data[1]=1;
		data[2]=2;
		data[3]=1;
		data[4]=3;
		data[5]=1;

		ReturnIfError(LoadParams(TC_NOW, 5, 6, data));
	// disable stuffing
		data[0]=5;
		data[1]=0;

		ReturnIfError(LoadParams(TC_NOW, 3, 2, data));
	}
// FIXED QUANT

	if (Dev->CaptureInfo.FixedQuant)
	{
		data[0]=4;
		// vee model quality.vee
		data[1]=(int)CalcQuant(Dev->CaptureInfo.Quality); 

		ReturnIfError(LoadParams(TC_NOW, 3, 2, data));
	}
	if (Dev->CaptureInfo.ShiftUp!=0)
	{
		data[0]=3;
		data[1]=(DWORD)(-Dev->CaptureInfo.ShiftUp);

		ReturnIfError(LoadParams(TC_NOW, 2, 2, data));
	}

	return(VT_NO_ERROR);
}

/***********************************************************************
 *
 * initialize CL4000 array for SIF processing.
 *
 ***********************************************************************/
//#pragma optimize( "", off )

 
 ECODE CLbrCapture::init_SIF()

{
	PREGS Regs=GET_STRUCT_PTR(Dev);
    int bps_scaled, bits_per_frame, vbv_delay, i;
	int vbvsize, initvbvfullness;
/* initialize areas in DRAM */

#if 0
    /* generate default q matrix */
	// do not use PEB 1/13/97
		qmatrix( QUANT_TABLES_INTRA, TRUE, TRUE);
		qmatrix( DQUANT_TABLES_INTRA, FALSE, TRUE);
//		qmatrix( ALT_QUANT_TABLES_NONINTRA, TRUE, FALSE);
//		qmatrix( ALT_DQUANT_TABLES_NONINTRA, FALSE, FALSE);
		qmatrix( QUANT_TABLES_NONINTRA, TRUE, FALSE);
		qmatrix( DQUANT_TABLES_NONINTRA, FALSE, FALSE);
#else
	Dev->LoadFileBinSwap(VR_DRAM,"qtables.int", QUANT_TABLES_INTRA, QUANT_TABLES_INTRA + TOTAL_QUANT_TABLE_SIZE - 1, 0);
    Dev->LoadFileBinSwap(VR_DRAM, "qtables.non", QUANT_TABLES_NONINTRA, QUANT_TABLES_NONINTRA + TOTAL_QUANT_TABLE_SIZE - 1, 0);
    Dev->LoadFileBinSwap(VR_DRAM, "dqtables.int", DQUANT_TABLES_INTRA, DQUANT_TABLES_INTRA + TOTAL_QUANT_TABLE_SIZE - 1, 0);
    Dev->LoadFileBinSwap(VR_DRAM, "dqtables.non", DQUANT_TABLES_NONINTRA, DQUANT_TABLES_NONINTRA + TOTAL_QUANT_TABLE_SIZE - 1, 0);
    /* Load quant tables for lowbitrate mode */
    Dev->LoadFileBinSwap(VR_DRAM, "qtables_ff.int", ALT_QUANT_TABLES_INTRA, ALT_QUANT_TABLES_INTRA + TOTAL_QUANT_TABLE_SIZE - 1, 0);
    Dev->LoadFileBinSwap(VR_DRAM,"qtables_ff.non", ALT_QUANT_TABLES_NONINTRA, ALT_QUANT_TABLES_NONINTRA + TOTAL_QUANT_TABLE_SIZE - 1, 0);
    Dev->LoadFileBinSwap(VR_DRAM, "dqtables_ff.int", ALT_DQUANT_TABLES_INTRA, ALT_DQUANT_TABLES_INTRA + TOTAL_QUANT_TABLE_SIZE - 1, 0);
    Dev->LoadFileBinSwap(VR_DRAM, "dqtables_ff.non", ALT_DQUANT_TABLES_NONINTRA, ALT_DQUANT_TABLES_NONINTRA + TOTAL_QUANT_TABLE_SIZE - 1, 0);
#endif


//    VT_LoadFileBin(Dev, VR_DRAM,"qtab.bii",  QUANT_TABLES_INTRA, QUANT_TABLES_INTRA + TOTAL_QUANT_TABLE_SIZE - 1, 0);

  /* for low bitrate operation this should be done using loadQmatrix() */

    VT_LoadFileBin(Dev, VR_DRAM,"cos_tbl.bii",  COS_TABLE, COS_TABLE + 32 - 1, 0);
    VT_LoadFileBin(Dev, VR_DRAM,"vlc_tbl.bii",  VLC_TABLES, VLC_TABLES + 0x2c0 - 1, 0);
    VT_LoadFileBin(Dev, VR_DRAM,"f_vbfvbs.bii", DA_F_VBS_VBF, DA_F_VBS_VBF + 0x200 - 1, 0);


/* write bitrate parameters into DRAM  - MUST be done after code is loaded! */

    bps_scaled = Dev->CaptureInfo.DataRate/400;
														//note framerate should be 29.97
    bits_per_frame = (Dev->CaptureInfo.DataRate *100) / Dev->CaptureInfo.FrameRate ;
    vbv_delay = 943718400 / (Dev->CaptureInfo.DataRate/100);
	vbvsize=327680;
	initvbvfullness=320000>>1;
	if (!Dev->CaptureInfo.IBP)
	{
		initvbvfullness=640000>>1;
		vbvsize=vbvsize*2;
	}


    dramWrite(Regs , BITS_PER_SEC_DIV_400 >> 2 , bps_scaled);
    dramWrite(Regs , ONE_OVER_VBV_SIZE >> 2    , VBV_RECIP);
    dramWrite(Regs , INIT_VBV_FULLNESS >> 2    , initvbvfullness);
    dramWrite(Regs , HDR_VBV_SIZE_LOC >> 2     , vbvsize);
    dramWrite(Regs , VBV_DLY_SCALE_LOC >> 2    , vbv_delay);
    dramWrite(Regs , AVG_BITS_PER_FRAME >> 2   , bits_per_frame);
	dramWrite(Regs, IONLY_FLAG >>2, (Dev->CaptureInfo.IBP? 0:1));


    /* Initialize the interrupt enable locations */
    for (i=0; i< INT_AREA_SIZE; i++)
     dramWrite(Regs , (INT_AREA + i), int_area_value);

    /* Setup Memory locations for scr register */
    dramWrite(Regs , DA_SCR_REG_ADDR>>2, scr_reg_value); 

    /* Setup Memory locations for audio_start_register */
    dramWrite(Regs , DA_AUDIO_START_REG>>2, as_reg_value); 

    /* Setup Memory location for pts register */
    dramWrite(Regs , DA_PTS_REG_ADDR>>2, pts_reg_value); 

    /* Setup Memory location for wide video A register */
       dramWrite(Regs, DA_VIDA_BUS_WIDTH >> 2, 0x10);

    /* Setup Memory location for wide video B register */
       dramWrite(Regs, DA_VIDB_BUS_WIDTH >> 2, 0x10);

    /* Setup Memory location for vsync polarity register */
    if (0)
       dramWrite(Regs, DA_VSYNC_POLARITY >> 2, 0x1);
    else
       dramWrite(Regs, DA_VSYNC_POLARITY >> 2, 0x0);

    /* Initialize ME MemCtl */
    GBusWrite(GBUS_MEMEMCTL,0xc0000040);    

    /* Clear Host semaphore bit */
    GBusWrite(GBUS_SEMA,0x0);               
   
    /* Disable all INTs. */    
    WBusWrite(Regs,WBUS_INTCTL,0xfffefffe);      
   
    /* Reset Mul_Cntl Reg (clip 9) */
    WBusWrite(Regs,WBUS_MULCTL,0x4);             
    

     dramWrite(Regs , DA_FEATURE_FLAGS >> 2, 0); 
   
	/*
     * Enable DMA Unit 
     * (Normal, Interleaved, 8 or 9-bit page, Refr Count = 0x100) 
     */
      //GBusWrite(GBUS_DMAMODE,0xc9000000 | DMA_MODE_9_CHP); 
      GBusWrite(GBUS_DMAMODE,VIDTOOL_MEMCONFIG); 

/* use Scorpio dma_mode configuration */

    /*-------------------------------------------------------------
     *  Configure dma_mode register of VRP to set memory type 
     *  8x10, 9x9, 10x8, memory organization (interleaved), and
     *  refresh count = 0x100.
     *  This code reads the SCV register of the
     *  scorpio/jupiter board to determine dram type and sets VRP
     *  accordingly.  Note customers can hard-code this if necessary.
     */
    /*-------------------------------------------------------------*/
	DWORD tmp;
    /* on Scorpio do not allow output bank switching mode */
    dramRead(Regs , DA_FEATURE_FLAGS >> 2, &tmp);
    tmp = tmp & (~FEATURE_OBANK_ENABLE);
    dramWrite(Regs , DA_FEATURE_FLAGS >> 2, tmp);
	dramWrite(Regs , DA_BANK1B_SIZE >> 2, 0);
    dramWrite(Regs , DA_BANK2B_SIZE >> 2, 0);


    /* set obits field selection flag */
    dramRead(Regs , DA_FEATURE_FLAGS >> 2, &tmp); 
    tmp = tmp  | FEATURE_OBITS_FIELD;	/* this sets obits 2nd field output */
    dramWrite(Regs , DA_FEATURE_FLAGS >> 2, tmp); 
   


    /* turn on VLC unit (MPEG, Intra, empty PACK & ZZ) */
    GBusWrite(GBUS_VLCCTL,0x00080105);
    GBusWrite(GBUS_VLCCTL,0xc0080105);

#ifdef SC_4010	/* E2: Use Autostart for ZZ xfer */
    /* Reset Vlc_Start register and set AutoStart feature ON */
    GBusWrite(GBUS_VLCSTART,0x4000);
#else   
    /* Reset Vlc_Start register. */
    GBusWrite(GBUS_VLCSTART,0x0);
#endif

    /* Zero out VLC Pack register, 
     * enable CPU module (reset Ghalt, Greset), set PIO as input, 
     */
    WBusWrite(Regs,WBUS_CPUPACK,0x0);
    WBusWrite(Regs,WBUS_CPUCTL,0xc0000000);

    /* enable smem, refresh and VLC */
    WBusWrite(Regs,WBUS_INTCTL,0x00010201);
 
    /* setup VLC DMA */
#if 1
    WBusWrite(Regs,WBUS_CHMEM5,((0x1ffu << 23) | (BITSTREAM_BASE_ADDR << 2) | 0x3)); 
#else
    WBusWrite(Regs,WBUS_CHMEM5,0xff900003);
#endif
	return (VT_NO_ERROR);
}



