// AudioLevel.cpp : implementation file
//
#include "stdafx.h"
#include "AudioLevel.h"
#include "mmsystem.h"													//for CAudioLevel

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAudioLevel
const unsigned int	CAudioLevel::m_kMaximumVOLUME = 65535;
const CString	CAudioLevel::m_kUtilityNAME = "SndVol32.EXE";
const CString	CAudioLevel::m_kUtilityPATH = "";

CAudioLevelSTATUS	CAudioLevel::m_Status = CAudioLevelSTATUSOK;

PROCESS_INFORMATION	CAudioLevel::m_ProcessInformation = {NULL, NULL, 0, 0};
CString	CAudioLevel::m_UtilityName = CAudioLevel::m_kUtilityNAME;
CString	CAudioLevel::m_UtilityPath = CAudioLevel::m_kUtilityPATH;

CToolHelp::CToolHelp()
{	
	m_pCreateToolhelp32Snapshot = NULL; 
	m_pModule32First  = NULL; 
	m_pModule32Next   = NULL; 
	m_pProcess32First = NULL; 
	m_pProcess32Next  = NULL; 
	m_pThread32First  = NULL; 
	m_pThread32Next   = NULL; 
	m_bHaveToolHelp   = NULL;
	
	m_bHaveToolHelp = InitToolhelp32();
}

BOOL CALLBACK CToolHelp::EnumThreadWindowsFunc (HWND hWnd, LPARAM lParam)
{
	CToolHelp *pToolHelp = (CToolHelp *)lParam;

	// Windows is just way too goofy! After going throw the hastle of walking  
	// through all the running processes to find the app we want, and then 
	// walking through all the topmost window until we get the one associated
	// with that process, we still may not have the main window for the app in
	// question. The top level window may be the app button on the taskbar if
	// the window is minimized.  IsIconic() and OpenIcon() just doesn't work
	// on this window.  So, we enumerate all the windows in the app thread and
	// do an IsIconic() on them. If we hit one that is iconic (minimized) we
	// know (or hope!!!) that this is the handle to the minimized app and set 
	// the member variable m_hWndTargetApp accordingly.
	if (IsIconic(hWnd)) 
	{
		pToolHelp->m_hWndTargetApp = hWnd;		
		return FALSE;
	}

	return TRUE;
}


BOOL CALLBACK CToolHelp::EnumWindowFunc (HWND hWnd, LPARAM lParam)
{
	CToolHelp *pToolHelp = (CToolHelp *)lParam;
	DWORD dwProcessId;

	// This method is a callback for EnumerateWindows. It checks the process
	// ID of each top level window of currently running apps and compares it 
	// to the member variable m_dwAppProcessID. If a match is found, it sets
	// the member variable m_hWndTargetApp to the hWnd associated with the
	// process.
	DWORD dwThreadId = GetWindowThreadProcessId(hWnd, &dwProcessId);
	if (dwProcessId == pToolHelp->m_dwAppProcessID)
	{
		pToolHelp->m_hWndTargetApp = hWnd;		
		EnumThreadWindows(dwThreadId, EnumThreadWindowsFunc, (LPARAM)pToolHelp);

		return FALSE;
	}

	return TRUE;
}						

HWND CToolHelp::GetProcessWindow (DWORD dwProcessId)
{
	// Initialize hWnd to NULL for a clean exit if a error is encountered
	// or the specified application is not running
	m_hWndTargetApp = NULL;

	// Enumerate all top level windows until we find one the matches the
	// give process id.
	m_dwAppProcessID = dwProcessId;
	EnumWindows(EnumWindowFunc, (LPARAM)this);

	return m_hWndTargetApp;
}

HWND CToolHelp::GetApplicationWindow (LPCTSTR p_szAppName)
{ 
    HANDLE hProcessSnap = NULL; 
    PROCESSENTRY32 pe32 = {0}; 

	// Initialize hWnd to NULL for a clean exit if a error is encountered
	// or the specified application is not running
	m_hWndTargetApp = NULL;

	// verify toolhelp is available
	if (!m_bHaveToolHelp) 
		return (NULL);

    //  Take a snapshot of all processes currently in the system. 
    hProcessSnap = m_pCreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0); 
    if (hProcessSnap == (HANDLE)-1) 
        return (NULL); 
 
    //  Fill in the size of the structure before using it. 
    pe32.dwSize = sizeof(PROCESSENTRY32); 
 
    //  Walk the snapshot of the processes, and for each process, get 
    //  information to compare. 
    if (m_pProcess32First(hProcessSnap, &pe32)) 
	{ 
        BOOL bGotModule = FALSE; 
		MODULEENTRY32 me32 = {0}; 
 
        do 
		{ 
            bGotModule = GetProcessModule(pe32.th32ProcessID, 
                pe32.th32ModuleID, &me32, sizeof(MODULEENTRY32)); 
            if (bGotModule) { 
                HANDLE hProcess; 
 
                // Get the actual priority class. 
                hProcess = OpenProcess (PROCESS_ALL_ACCESS, 
                    FALSE, pe32.th32ProcessID); 
                CloseHandle (hProcess); 
 
                // Compare the process's module name and exe path to the one supplied as
				// an input parameter. 
				if (!lstrcmpi (p_szAppName, me32.szModule) || !lstrcmpi (p_szAppName, me32.szExePath)) 
				{
					m_dwAppProcessID = me32.th32ProcessID;
					EnumWindows(EnumWindowFunc, (LPARAM)this);
					break;
				}				 
            } 
        } 

        while (m_pProcess32Next(hProcessSnap, &pe32)); 
    } 
 
    // Do not forget to clean up the snapshot object. 
    CloseHandle (hProcessSnap); 
    return m_hWndTargetApp; 
}
		   					

BOOL CToolHelp::InitToolhelp32 () 
{ 
    BOOL   bRet  = FALSE; 
    HINSTANCE hKernel = NULL; 
 
    // Obtain the module handle of the kernel to retrieve addresses of 
    // the tool helper functions. 
    hKernel = GetModuleHandle("KERNEL32.DLL"); 
 
    if (hKernel)
	{ 
        m_pCreateToolhelp32Snapshot = 
            (CREATESNAPSHOT)GetProcAddress(hKernel, 
            "CreateToolhelp32Snapshot"); 
 
        m_pModule32First  = (MODULEWALK)GetProcAddress(hKernel, 
            "Module32First"); 
        m_pModule32Next   = (MODULEWALK)GetProcAddress(hKernel, 
            "Module32Next"); 

        m_pProcess32First = (PROCESSWALK)GetProcAddress(hKernel, 
            "Process32First"); 
        m_pProcess32Next  = (PROCESSWALK)GetProcAddress(hKernel, 
            "Process32Next"); 
 
        m_pThread32First  = (THREADWALK)GetProcAddress(hKernel, 
            "Thread32First"); 
        m_pThread32Next   = (THREADWALK)GetProcAddress(hKernel, 
            "Thread32Next"); 
 
        // All addresses must be non-NULL to be successful. 
        // If one of these addresses is NULL, one of 
        // the needed lists cannot be walked. 
        bRet =  m_pModule32First && m_pModule32Next  && m_pProcess32First && 
                m_pProcess32Next && m_pThread32First && m_pThread32Next && 
                m_pCreateToolhelp32Snapshot; 
    } 
    else 
        bRet = FALSE; // could not even get the module handle of kernel 
 
    return bRet; 
} 



BOOL CToolHelp::GetProcessModule (DWORD dwPID, DWORD dwModuleID, 
        LPMODULEENTRY32 lpMe32, DWORD cbMe32) 
{ 
    BOOL          bRet        = FALSE; 
    BOOL          bFound      = FALSE; 
    HANDLE        hModuleSnap = NULL; 
    MODULEENTRY32 me32        = {0}; 
 
	// Returns TRUE if there is information about the specified module or  
	//   FALSE if it could not enumerate the modules in the process or 
	//   the module is not found in the process. 
	// dwPID - identifier of the process that owns the module to 
	//   retrieve information about. 
	// dwModuleID - tool help identifier of the module within the 
	//   process 
	// lpMe32 - structure to return data about the module 
	// cbMe32 - size of the buffer pointed to by lpMe32 (to ensure 
	//   the buffer is not over filled) 

	// verify toolhelp is available
	if (!m_bHaveToolHelp) 
		return FALSE;

    // Take a snapshot of all modules in the specified process. 
    hModuleSnap = m_pCreateToolhelp32Snapshot(TH32CS_SNAPMODULE, dwPID); 
    if (hModuleSnap == (HANDLE)-1) 
        return (FALSE); 
 
    // Fill the size of the structure before using it. 
    me32.dwSize = sizeof(MODULEENTRY32); 
 
    // Walk the module list of the process, and find the module of 
    // interest. Then copy the information to the buffer pointed 
    // to by lpMe32 so that it can be returned to the caller. 
    if (m_pModule32First(hModuleSnap, &me32)) 
	{ 
        do 
		{ 
            if (me32.th32ModuleID == dwModuleID) 
			{ 
                CopyMemory (lpMe32, &me32, cbMe32); 
                bFound = TRUE; 
            } 
        } 
        while (!bFound && m_pModule32Next(hModuleSnap, &me32)); 
 
        bRet = bFound;   // if this sets bRet to FALSE, dwModuleID 
                         // no longer exists in specified process 
    } 
    else 
        bRet = FALSE;           // could not walk module list 
 
    // Do not forget to clean up the snapshot object. 
    CloseHandle (hModuleSnap); 
 
    return (bRet); 
} 
 
CAudioLevelSTATUS CAudioLevel::Name(CString p_Name)
{
	// Ensure that the supplied name is not empty.
	if (p_Name.IsEmpty())												//err?
		Status(CAudioLevelSTATUSNAMEEMPTY);
	else
	{
		// Copy the name.
		m_UtilityName = p_Name;
		Status(CAudioLevelSTATUSOK);
	}
	return m_Status;
}

CAudioLevelSTATUS CAudioLevel::Path(CString p_Path)
{
	const TCHAR	BACKSLASHCHAR = '\\';									//bk slash in chr form
	const CString	BACKSLASHSTRING = "\\";								//bk slash in str form

	// Copy the path and ensure that it ends with a back slash.
	m_UtilityName = p_Path;
	if (m_UtilityName[m_UtilityName.GetLength() - 1] != BACKSLASHCHAR)	//no bk slash?
		m_UtilityName += BACKSLASHSTRING;
	Status(CAudioLevelSTATUSOK);
	return m_Status;
}

unsigned int CAudioLevel::PlayingVolume()
{
	unsigned int	count = mixerGetNumDevs();							//# of mxrs
	int	volume = 32767;													//vol

	if (count == 0)														//no mxrs?
		Status(CAudioLevelSTATUSMIXERS);
	else
	{
		MIXERCAPS*	capabilities = (MIXERCAPS*)new char[count * sizeof(MIXERCAPS)];	//mxr desc
		MMRESULT	result = MMSYSERR_NOERROR;							//sts

		// The mixer's capabilities are not used directly, but they
		// must exist.
		if (capabilities == NULL)										//no mem?
			Status(CAudioLevelSTATUSMEMORY);
		else
		{
			// Examine each mixer and then release the storage
			// allocated to contain the mixer capabilities. Internal
			// errors are "remembered" by the setting of the status.
			// However, the loop continues. The last set status is the
			// one which is used.
			for (int i = 0; i < (int)count; ++i)						//for all mxrs...
			{
				MIXERLINE	mixerLine;									//mxr line

				// For each mixer, find the line which controls the
				// volume to the speakers.
				mixerLine.cbStruct = sizeof(MIXERLINE);
				mixerLine.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_SPEAKERS;
				result = mixerGetLineInfo((HMIXEROBJ)i,
										  &mixerLine,
										  MIXER_GETLINEINFOF_COMPONENTTYPE | MIXER_OBJECTF_MIXER);
				if (result != MMSYSERR_NOERROR)							//err?
					Status(CAudioLevelSTATUSLINE);
				else
				{
					MIXERCONTROL	control;							//mxr ctl
					MIXERLINECONTROLS	lineControls;					//line ctl

					// The line has been found. Find the control which
					// changes the volume.
					lineControls.cbStruct = sizeof(MIXERLINECONTROLS);
					lineControls.dwLineID = mixerLine.dwLineID;
					lineControls.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
					lineControls.cControls = 1;
					lineControls.cbmxctrl = sizeof(MIXERCONTROL);
					lineControls.pamxctrl = &control;
					result = mixerGetLineControls((HMIXEROBJ)i,
												  &lineControls,
												  MIXER_GETLINECONTROLSF_ONEBYTYPE | MIXER_OBJECTF_MIXER);
					if (result != MMSYSERR_NOERROR)						//err?
						Status(CAudioLevelSTATUSCONTROLS);
					else
					{
						MIXERCONTROLDETAILS	mixerDetails;				//mxr dtls
						DWORD	size = 0;								//siz

						// The volume line control has been found.
						// Find and fetch the detailed information for
						// the line.
						mixerDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
						mixerDetails.dwControlID = control.dwControlID;
						mixerDetails.cChannels = 1;

						if (control.fdwControl & MIXERCONTROL_CONTROLTYPE_CUSTOM)	//reset?
							mixerDetails.cChannels = 0;
						else
						if (control.fdwControl & MIXERCONTROL_CONTROLF_UNIFORM)	//reset?
							mixerDetails.cChannels = 1;
						else;
						mixerDetails.hwndOwner = NULL;
						if (control.cMultipleItems & MIXERCONTROL_CONTROLF_MULTIPLE)
							mixerDetails.cMultipleItems = control.cMultipleItems;
						else
							mixerDetails.cMultipleItems = 0;
						mixerDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);	//fader vol
						if (control.fdwControl & MIXERCONTROL_CONTROLF_MULTIPLE)	//mult?
						{
							size = mixerDetails.cChannels * mixerDetails.cMultipleItems;
							size *= sizeof(MIXERCONTROLDETAILS_UNSIGNED);
						}
						else
							size = mixerDetails.cChannels * sizeof(MIXERCONTROLDETAILS_UNSIGNED);
						mixerDetails.paDetails = (LPVOID)new char[size];
						if (mixerDetails.paDetails == NULL)				//out of mem?
							Status(CAudioLevelSTATUSMEMORY);
						else
						{
							// Fetch the detailed information for the
							// line, fetch the volume, and then
							// release the storage allocated for the
							// detailed information.
							result = mixerGetControlDetails((HMIXEROBJ)i,
															&mixerDetails,
															MIXER_GETCONTROLDETAILSF_VALUE | MIXER_OBJECTF_MIXER);
							if (result != MMSYSERR_NOERROR)				//err?
								Status(CAudioLevelSTATUSDETAILS);
							else
							{
								// Fetch the volume and set the status.
								LPMIXERCONTROLDETAILS_UNSIGNED	details = (LPMIXERCONTROLDETAILS_UNSIGNED)mixerDetails.paDetails;
								volume = details->dwValue;
								Status(CAudioLevelSTATUSOK);
							}
							delete[] mixerDetails.paDetails;
							mixerDetails.paDetails = NULL;
						}
					}
				}
			}
			delete[] capabilities;
			capabilities = NULL;
		}
	}
	return volume;
}

CAudioLevelSTATUS CAudioLevel::PlayingVolume(unsigned int p_Volume)
{
	unsigned int	count = mixerGetNumDevs();							//# of mxrs

	if (count == 0)														//no mxrs?
		Status(CAudioLevelSTATUSMIXERS);
	else
	{
		MIXERCAPS*	capabilities = (MIXERCAPS*)new char[count * sizeof(MIXERCAPS)];	//mxr desc
		MMRESULT	result = MMSYSERR_NOERROR;							//sts

		// The mixer's capabilities are not used directly, but they
		// must exist.
		if (capabilities == NULL)										//no mem?
			Status(CAudioLevelSTATUSMEMORY);
		else
		{
			// Examine each mixer and then release the storage
			// allocated to contain the mixer capabilities. Internal
			// errors are "remembered" by the setting of the status.
			// However, the loop continues. The last set status is the
			// one which is used.
			for (int i = 0; i < (int)count; ++i)						//for all mxrs...
			{
				MIXERLINE	mixerLine;									//mxr line

				// For each mixer, find the line which controls the
				// volume to the speakers.
				mixerLine.cbStruct = sizeof(MIXERLINE);
				mixerLine.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_SPEAKERS;
				result = mixerGetLineInfo((HMIXEROBJ)i,
										  &mixerLine,
										  MIXER_GETLINEINFOF_COMPONENTTYPE | MIXER_OBJECTF_MIXER);
				if (result != MMSYSERR_NOERROR)							//err?
					Status(CAudioLevelSTATUSLINE);
				else
				{
					MIXERLINECONTROLS	lineControls;					//line ctl
					MIXERCONTROL	control;							//mxr ctl

					// The line has been found. Find the control which
					// changes the volume.
					lineControls.cbStruct = sizeof(MIXERLINECONTROLS);
					lineControls.dwLineID = mixerLine.dwLineID;
					lineControls.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
					lineControls.cControls = 1;
					lineControls.cbmxctrl = sizeof(MIXERCONTROL);
					lineControls.pamxctrl = &control;
					result = mixerGetLineControls((HMIXEROBJ)i,
												  &lineControls,
												  MIXER_GETLINECONTROLSF_ONEBYTYPE | MIXER_OBJECTF_MIXER);
					if (result != MMSYSERR_NOERROR)						//err?
						Status(CAudioLevelSTATUSCONTROLS);
					else
					{
						MIXERCONTROLDETAILS	mixerDetails;				//mxr dtls
						DWORD	size = 0;								//siz

						// The volume line control has been found.
						// Find and fetch the detailed information for
						// the line.
						mixerDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
						mixerDetails.dwControlID = control.dwControlID;
						mixerDetails.cChannels = 1;

						if (control.fdwControl & MIXERCONTROL_CONTROLTYPE_CUSTOM)	//reset?
							mixerDetails.cChannels = 0;
						else
						if (control.fdwControl & MIXERCONTROL_CONTROLF_UNIFORM)	//reset?
							mixerDetails.cChannels = 1;
						else;
						mixerDetails.hwndOwner = NULL;
						if (control.cMultipleItems & MIXERCONTROL_CONTROLF_MULTIPLE)
							mixerDetails.cMultipleItems = control.cMultipleItems;
						else
							mixerDetails.cMultipleItems = 0;
						mixerDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);	//fader vol
						if (control.fdwControl & MIXERCONTROL_CONTROLF_MULTIPLE)	//mult?
						{
							size = mixerDetails.cChannels * mixerDetails.cMultipleItems;
							size *= sizeof(MIXERCONTROLDETAILS_UNSIGNED);
						}
						else
							size = mixerDetails.cChannels * sizeof(MIXERCONTROLDETAILS_UNSIGNED);
						mixerDetails.paDetails = (LPVOID)new char[size];
						if (mixerDetails.paDetails == NULL)				//out of mem?
							Status(CAudioLevelSTATUSMEMORY);
						else
						{
							// Fetch the detailed information for the
							// line, set the volume, and then release
							// the storage allocated for the detailed
							// information.
							result = mixerGetControlDetails((HMIXEROBJ)i,
															&mixerDetails,
															MIXER_GETCONTROLDETAILSF_VALUE | MIXER_OBJECTF_MIXER);
							if (result != MMSYSERR_NOERROR)				//err?
								Status(CAudioLevelSTATUSDETAILS);
							else
							{
								// Set the volume and set the status.
								LPMIXERCONTROLDETAILS_UNSIGNED	details = (LPMIXERCONTROLDETAILS_UNSIGNED)mixerDetails.paDetails;
								details->dwValue = p_Volume;
								result = mixerSetControlDetails((HMIXEROBJ)i,
																&mixerDetails,
																MIXER_SETCONTROLDETAILSF_VALUE | MIXER_OBJECTF_MIXER);
								if (result != MMSYSERR_NOERROR)			//err?
									Status(CAudioLevelSTATUSSET);
								else
									Status(CAudioLevelSTATUSOK);
							}
							delete[] mixerDetails.paDetails;
							mixerDetails.paDetails = NULL;
						}
					}
				}
			}
			delete[] capabilities;
			capabilities = NULL;
		}
	}
	return m_Status;
}

unsigned int CAudioLevel::RecordingPeak()
{
	unsigned int	count = mixerGetNumDevs();							//# of mxrs
	unsigned int	peak = 0;											//peak

	if (count == 0)														//no mxrs?
		Status(CAudioLevelSTATUSMIXERS);
	else
	{
		MIXERCAPS*	capabilities = (MIXERCAPS*)new char[count * sizeof(MIXERCAPS)];	//mxr desc
		MMRESULT	result = MMSYSERR_NOERROR;							//sts

		// The mixer's capabilities are not used directly, but they
		// must exist.
		if (capabilities == NULL)										//no mem?
			Status(CAudioLevelSTATUSMEMORY);
		else
		{
			// Examine each mixer and then release the storage
			// allocated to contain the mixer capabilities. Internal
			// errors are "remembered" by the setting of the status.
			// However, the loop continues. The last set status is the
			// one which is used.
			for (int i = 0; i < (int)count; ++i)						//for all mxrs...
			{
				MIXERLINE	mixerLine;									//mxr line

				// For each mixer, find the line which controls the
				// volume to the wave in device.
				mixerLine.cbStruct = sizeof(MIXERLINE);
				mixerLine.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_WAVEIN;

				result = mixerGetLineInfo((HMIXEROBJ)i,
										  &mixerLine,
										  MIXER_GETLINEINFOF_COMPONENTTYPE | MIXER_OBJECTF_MIXER);
				if (result != MMSYSERR_NOERROR)							//err?
					Status(CAudioLevelSTATUSLINE);
				else
				{
					MIXERLINECONTROLS	lineControls;					//line ctl
					MIXERCONTROL	control;							//mxr ctl

					// The line has been found. Find the control which
					// measures the peak recording level.
					lineControls.cbStruct = sizeof(MIXERLINECONTROLS);
					lineControls.dwLineID = mixerLine.dwLineID;
					lineControls.dwControlType = MIXERCONTROL_CONTROLTYPE_PEAKMETER;
					lineControls.cControls = 1;
					lineControls.cbmxctrl = sizeof(MIXERCONTROL);
					lineControls.pamxctrl = &control;
					result = mixerGetLineControls((HMIXEROBJ)i,
												  &lineControls,
												  MIXER_GETLINECONTROLSF_ONEBYTYPE | MIXER_OBJECTF_MIXER);
					if (result != MMSYSERR_NOERROR)						//err?
						Status(CAudioLevelSTATUSCONTROLS);
					else
					{
						MIXERCONTROLDETAILS	mixerDetails;				//mxr dtls
						DWORD	size = 0;								//siz

						// The peak meter control has been found.
						// Find and fetch the detailed information for
						// the line.
						mixerDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
						mixerDetails.dwControlID = control.dwControlID;
						mixerDetails.cChannels = 1;

						if (control.fdwControl & MIXERCONTROL_CONTROLTYPE_CUSTOM)	//reset?
							mixerDetails.cChannels = 0;
						else
						if (control.fdwControl & MIXERCONTROL_CONTROLF_UNIFORM)	//reset?
							mixerDetails.cChannels = 1;
						else;
						mixerDetails.hwndOwner = NULL;
						if (control.cMultipleItems & MIXERCONTROL_CONTROLF_MULTIPLE)
							mixerDetails.cMultipleItems = control.cMultipleItems;
						else
							mixerDetails.cMultipleItems = 0;
						mixerDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);	//fader vol
						if (control.fdwControl & MIXERCONTROL_CONTROLF_MULTIPLE)	//mult?
						{
							size = mixerDetails.cChannels * mixerDetails.cMultipleItems;
							size *= sizeof(MIXERCONTROLDETAILS_UNSIGNED);
						}
						else
							size = mixerDetails.cChannels * sizeof(MIXERCONTROLDETAILS_UNSIGNED);
						mixerDetails.paDetails = (LPVOID)new char[size];
						if (mixerDetails.paDetails == NULL)				//out of mem?
							Status(CAudioLevelSTATUSMEMORY);
						else
						{
							// Fetch the detailed information for the
							// line, fetch the peak, and then
							// release the storage allocated for the
							// detailed information.
							result = mixerGetControlDetails((HMIXEROBJ)i,
															&mixerDetails,
															MIXER_GETCONTROLDETAILSF_VALUE | MIXER_OBJECTF_MIXER);
							if (result != MMSYSERR_NOERROR)				//err?
								Status(CAudioLevelSTATUSDETAILS);
							else
							{
								// Fetch the volume and set the status.
								LPMIXERCONTROLDETAILS_UNSIGNED	details = (LPMIXERCONTROLDETAILS_UNSIGNED)mixerDetails.paDetails;
								peak = (unsigned int)abs((int)details->dwValue);
								Status(CAudioLevelSTATUSOK);
							}
							delete[] mixerDetails.paDetails;
							mixerDetails.paDetails = NULL;
						}
					}
				}
			}
			delete[] capabilities;
			capabilities = NULL;
		}
	}
	return peak;
}

unsigned int CAudioLevel::RecordingVolume()
{
	unsigned int	count = mixerGetNumDevs();							//# of mxrs
	unsigned int	volume = 32767;										//vol

	if (count == 0)														//no mxrs?
		Status(CAudioLevelSTATUSMIXERS);
	else
	{
		MIXERCAPS*	capabilities = (MIXERCAPS*)new char[count * sizeof(MIXERCAPS)];	//mxr desc
		MMRESULT	result = MMSYSERR_NOERROR;							//sts

		// The mixer's capabilities are not used directly, but they
		// must exist.
		if (capabilities == NULL)										//no mem?
			Status(CAudioLevelSTATUSMEMORY);
		else
		{
			// Examine each mixer and then release the storage
			// allocated to contain the mixer capabilities. Internal
			// errors are "remembered" by the setting of the status.
			// However, the loop continues. The last set status is the
			// one which is used.
			for (int i = 0; i < (int)count; ++i)						//for all mxrs...
			{
				MIXERLINE	mixerLine;									//mxr line

				// For each mixer, find the line which controls the
				// volume to wave in device.
				mixerLine.cbStruct = sizeof(MIXERLINE);
				mixerLine.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_WAVEIN;

				result = mixerGetLineInfo((HMIXEROBJ)i,
										  &mixerLine,
										  MIXER_GETLINEINFOF_COMPONENTTYPE | MIXER_OBJECTF_MIXER);
				if (result != MMSYSERR_NOERROR)							//err?
					Status(CAudioLevelSTATUSLINE);
				else
				{
					MIXERLINECONTROLS	lineControls;					//line ctl
					MIXERCONTROL	control;							//mxr ctl

					// The line has been found. Find the control which
					// changes the volume.
					lineControls.cbStruct = sizeof(MIXERLINECONTROLS);
					lineControls.dwLineID = mixerLine.dwLineID;
					lineControls.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
					lineControls.cControls = 1;
					lineControls.cbmxctrl = sizeof(MIXERCONTROL);
					lineControls.pamxctrl = &control;
					result = mixerGetLineControls((HMIXEROBJ)i,
												  &lineControls,
												  MIXER_GETLINECONTROLSF_ONEBYTYPE | MIXER_OBJECTF_MIXER);
					if (result != MMSYSERR_NOERROR)						//err?
						Status(CAudioLevelSTATUSCONTROLS);
					else
					{
						MIXERCONTROLDETAILS	mixerDetails;				//mxr dtls
						DWORD	size = 0;								//siz

						// The volume line control has been found.
						// Find and fetch the detailed information for
						// the line.
						mixerDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
						mixerDetails.dwControlID = control.dwControlID;
						mixerDetails.cChannels = 1;

						if (control.fdwControl & MIXERCONTROL_CONTROLTYPE_CUSTOM)	//reset?
							mixerDetails.cChannels = 0;
						else
						if (control.fdwControl & MIXERCONTROL_CONTROLF_UNIFORM)	//reset?
							mixerDetails.cChannels = 1;
						else;
						mixerDetails.hwndOwner = NULL;
						if (control.cMultipleItems & MIXERCONTROL_CONTROLF_MULTIPLE)
							mixerDetails.cMultipleItems = control.cMultipleItems;
						else
							mixerDetails.cMultipleItems = 0;
						mixerDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);	//fader vol
						if (control.fdwControl & MIXERCONTROL_CONTROLF_MULTIPLE)	//mult?
						{
							size = mixerDetails.cChannels * mixerDetails.cMultipleItems;
							size *= sizeof(MIXERCONTROLDETAILS_UNSIGNED);
						}
						else
							size = mixerDetails.cChannels * sizeof(MIXERCONTROLDETAILS_UNSIGNED);
						mixerDetails.paDetails = (LPVOID)new char[size];
						if (mixerDetails.paDetails == NULL)				//out of mem?
							Status(CAudioLevelSTATUSMEMORY);
						else
						{
							// Fetch the detailed information for the
							// line, fetch the volume, and then
							// release the storage allocated for the
							// detailed information.
							result = mixerGetControlDetails((HMIXEROBJ)i,
															&mixerDetails,
															MIXER_GETCONTROLDETAILSF_VALUE | MIXER_OBJECTF_MIXER);
							if (result != MMSYSERR_NOERROR)				//err?
								Status(CAudioLevelSTATUSDETAILS);
							else
							{
								// Fetch the volume and set the status.
								LPMIXERCONTROLDETAILS_UNSIGNED	details = (LPMIXERCONTROLDETAILS_UNSIGNED)mixerDetails.paDetails;
								volume = details->dwValue;
								Status(CAudioLevelSTATUSOK);
							}
							delete[] mixerDetails.paDetails;
							mixerDetails.paDetails = NULL;
						}
					}
				}
			}
			delete[] capabilities;
			capabilities = NULL;
		}
	}
	return volume;
}

CAudioLevelSTATUS CAudioLevel::RecordingVolume(unsigned int p_Volume)
{
	unsigned int	count = mixerGetNumDevs();							//# of mxrs

	if (count == 0)														//no mxrs?
		Status(CAudioLevelSTATUSMIXERS);
	else
	{
		MIXERCAPS*	capabilities = (MIXERCAPS*)new char[count * sizeof(MIXERCAPS)];	//mxr desc
		MMRESULT	result = MMSYSERR_NOERROR;							//sts

		// The mixer's capabilities are not used directly, but they
		// must exist.
		if (capabilities == NULL)										//no mem?
			Status(CAudioLevelSTATUSMEMORY);
		else
		{
			// Examine each mixer and then release the storage
			// allocated to contain the mixer capabilities. Internal
			// errors are "remembered" by the setting of the status.
			// However, the loop continues. The last set status is the
			// one which is used.
			for (int i = 0; i < (int)count; ++i)						//for all mxrs...
			{
				MIXERLINE	mixerLine;									//mxr line

				// For each mixer, find the line which controls the
				// volume to wave in device.
				mixerLine.cbStruct = sizeof(MIXERLINE);
				mixerLine.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_WAVEIN;
				mixerLine.Target.dwType = MIXERLINE_TARGETTYPE_WAVEIN;

				result = mixerGetLineInfo((HMIXEROBJ)i,
										  &mixerLine,
										  MIXER_GETLINEINFOF_COMPONENTTYPE | MIXER_OBJECTF_MIXER);
				if (result != MMSYSERR_NOERROR)							//err?
					Status(CAudioLevelSTATUSLINE);
				else
				{
					MIXERLINECONTROLS	lineControls;					//line ctl
					MIXERCONTROL	control;							//mxr ctl

					// The line has been found. Find the control which
					// changes the volume.
					lineControls.cbStruct = sizeof(MIXERLINECONTROLS);
					lineControls.dwLineID = mixerLine.dwLineID;
					lineControls.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
					lineControls.cControls = 1;
					lineControls.cbmxctrl = sizeof(MIXERCONTROL);
					lineControls.pamxctrl = &control;
					result = mixerGetLineControls((HMIXEROBJ)i,
												  &lineControls,
												  MIXER_GETLINECONTROLSF_ONEBYTYPE | MIXER_OBJECTF_MIXER);
					if (result != MMSYSERR_NOERROR)						//err?
						Status(CAudioLevelSTATUSCONTROLS);
					else
					{
						MIXERCONTROLDETAILS	mixerDetails;				//mxr dtls
						DWORD	size = 0;								//siz

						// The volume line control has been found.
						// Find and fetch the detailed information for
						// the line.
						mixerDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
						mixerDetails.dwControlID = control.dwControlID;
						mixerDetails.cChannels = 1;

						if (control.fdwControl & MIXERCONTROL_CONTROLTYPE_CUSTOM)	//reset?
							mixerDetails.cChannels = 0;
						else
						if (control.fdwControl & MIXERCONTROL_CONTROLF_UNIFORM)	//reset?
							mixerDetails.cChannels = 1;
						else;
						mixerDetails.hwndOwner = NULL;
						if (control.cMultipleItems & MIXERCONTROL_CONTROLF_MULTIPLE)
							mixerDetails.cMultipleItems = control.cMultipleItems;
						else
							mixerDetails.cMultipleItems = 0;
						mixerDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);	//fader vol
						if (control.fdwControl & MIXERCONTROL_CONTROLF_MULTIPLE)	//mult?
						{
							size = mixerDetails.cChannels * mixerDetails.cMultipleItems;
							size *= sizeof(MIXERCONTROLDETAILS_UNSIGNED);
						}
						else
							size = mixerDetails.cChannels * sizeof(MIXERCONTROLDETAILS_UNSIGNED);
						mixerDetails.paDetails = (LPVOID)new char[size];
						if (mixerDetails.paDetails == NULL)				//out of mem?
							Status(CAudioLevelSTATUSMEMORY);
						else
						{
							// Fetch the detailed information for the
							// line, set the volume, and then release
							// the storage allocated for the detailed
							// information.
							result = mixerGetControlDetails((HMIXEROBJ)i,
															&mixerDetails,
															MIXER_GETCONTROLDETAILSF_VALUE | MIXER_OBJECTF_MIXER);
							if (result != MMSYSERR_NOERROR)				//err?
								Status(CAudioLevelSTATUSDETAILS);
							else
							{
								// Set the volume and set the status.
								LPMIXERCONTROLDETAILS_UNSIGNED	details = (LPMIXERCONTROLDETAILS_UNSIGNED)mixerDetails.paDetails;
								details->dwValue = p_Volume;
								result = mixerSetControlDetails((HMIXEROBJ)i,
																&mixerDetails,
																MIXER_SETCONTROLDETAILSF_VALUE | MIXER_OBJECTF_MIXER);
								if (result != MMSYSERR_NOERROR)			//err?
									Status(CAudioLevelSTATUSSET);
								else
									Status(CAudioLevelSTATUSOK);
							}
							delete[] mixerDetails.paDetails;
							mixerDetails.paDetails = NULL;
						}
					}
				}
			}
			delete[] capabilities;
			capabilities = NULL;
		}
	}
	return m_Status;
}

CAudioLevelSTATUS CAudioLevel::Ready()
{
	if (m_ProcessInformation.dwProcessId != 0)							//exists?
		if (WaitForSingleObject(m_ProcessInformation.hProcess, 0) != WAIT_OBJECT_0)	//done?
			Status(CAudioLevelSTATUSRUNNING);
		else
			Status(CAudioLevelSTATUSOK);
	else
		Status(CAudioLevelSTATUSOK);
	return m_Status;
}

HWND CAudioLevel::GetUtilityWindow()
{
	CToolHelp ToolHelp;
	HWND hWnd = NULL;

	// if we've started the run utility, get the main window handle
	// of the process we started
	if (Ready() != CAudioLevelSTATUSOK)									 //running?
		hWnd = ToolHelp.GetProcessWindow (m_ProcessInformation.dwProcessId);
	else
		hWnd = ToolHelp.GetApplicationWindow (m_UtilityName);

	return hWnd;
}

CAudioLevelSTATUS CAudioLevel::Run()
{
	CToolHelp ToolHelp;
	HWND hWnd = NULL;

	// if we've started the run utility, get the main window handle
	// of the process we started
	if (Ready() != CAudioLevelSTATUSOK)									 //running?
		hWnd = ToolHelp.GetProcessWindow (m_ProcessInformation.dwProcessId);
	else
		hWnd = ToolHelp.GetApplicationWindow (m_UtilityName);

	// At this pooint if ther is an instance of the run utility, set it as 
	// the active window and bring it to the forground, els run the utility.
	if (hWnd != NULL) 
	{
		if (IsIconic(hWnd))
			OpenIcon(hWnd);
		else 
			SetForegroundWindow(hWnd);
	}
	else 
	{
		// Clear the process information just to make things look OK.
		memset(&m_ProcessInformation, 0, sizeof(PROCESS_INFORMATION));
		Status(RunUtility());
	} 
	
	return m_Status;
}

CAudioLevelSTATUS CAudioLevel::RunUtility()
{
	STARTUPINFO	startInformation;										//start info

	ASSERT(m_ProcessInformation.dwProcessId == 0);

	memset(&startInformation, 0, sizeof(STARTUPINFO));
	startInformation.cb = sizeof(STARTUPINFO);
	startInformation.dwFlags = STARTF_FORCEONFEEDBACK;

	if (!CreateProcess(NULL,
					  m_UtilityName.GetBuffer(m_UtilityName.GetLength() + 1),
					  NULL,
					  NULL,
					  FALSE,
					  0,//DETACHED_PROCESS,
					  NULL,
					  NULL,
					  &startInformation,
					  &m_ProcessInformation))							//err?
	{
		memset(&m_ProcessInformation, 0, sizeof(PROCESS_INFORMATION));
		Status(CAudioLevelSTATUSRUN);
	}
	else
		Status(CAudioLevelSTATUSOK);

	m_UtilityName.ReleaseBuffer();

	return m_Status;
}

CAudioLevelSTATUS CAudioLevel::Status(CAudioLevelSTATUS p_Status)
{
	m_Status = p_Status;

	// This is a good place for an
	// ASSERT(m_Status != CAudioLevelSTATUSOK).
	return m_Status;
}
