// bwdialog.h : main header file for the BWDIALOG DLL
//
#define HIDC_MYBASE				 0x10000

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"		// main symbols


/////////////////////////////////////////////////////////////////////////////
// CRegistrySettings
// See bwdialog.cpp for the implementation of this class
//

class CRegistrySettings : public CObject 
{

public:
	CRegistrySettings();
	~CRegistrySettings();

    void CRegistrySettings::VerifyAudioSettings();

	UINT CaptureInitialize(BW_BOARDINFO *pInfo, PVC_PROFILE_INFO pProf);
	UINT RecodeInitialize();
	UINT Terminate();

	// Read / Write Settings based on compression/capture target string
    void ReadTargetSettings(CString sTarget);
    void WriteTargetSettings(CString sTarget);

	BOOL ReadEncodeType(LPCTSTR lpszEncodeType);
							   
	void ValidateCapturePreferences (BW_CAPTURE_PREFERENCES *pOptions);
	BOOL ValidatePlayBackTarget (LPCTSTR Target);
	BOOL ValidatePlayBackTarget (LPCTSTR Target, LPCTSTR EncodeType);
	BOOL ValidateEncodeType (LPCTSTR Target);
	void ValidateCompressionPreferences(BW_RECODE_PREFERENCES *pOptions);
	BOOL UpdateEncodeType (LPCTSTR Target);

	UINT GetSetting( UINT nSection, UINT nSetting );
	UINT GetSetting(CString sTarget,UINT nSetting,UINT nDefault);

	CString GetStringSetting( UINT nSection, UINT nSetting );
	CString GetStringSetting( UINT nSection, CString sSetting );
	CString GetStringSetting( CString sTarget, UINT nSetting );

	UINT SetSetting( UINT nSection, UINT nSetting, int nValue );
	UINT SetSetting( CString sSection, UINT nSetting, int nValue );

	UINT SetStringSetting( UINT nSection, UINT nSetting, CString sValue );
	UINT SetStringSetting( UINT nSection, CString sSetting, CString sValue );
	UINT SetStringSetting( CString sSection, UINT nSetting, CString sValue );

	UINT CreateDefaultSettings();
    UINT CreateDefaultSetting(UINT nSetting, UINT nValue);
    UINT CreateDefaultStringSetting(UINT nSetting, CString sValue);
    UINT GetInitialSetting(UINT nSetting);
    CString GetInitialStringSetting(UINT nSetting);

	UINT CreateInitialTargetSetting(CString Target, UINT nSetting); 
	UINT CreateInitialTargetStringSetting(CString Target, UINT nSetting); 

	void ReadSettings(char *sTarget);


private:

//data

	UINT m_nAutoDetect;
	UINT m_nNTSC;
	UINT m_nComposite;
	
	UINT m_nAudioEnable;
	UINT m_nAudioMode;
	UINT m_nAudioRate;
	
	UINT m_nHue;
	UINT m_nSaturation;
	UINT m_nIntensity;
	UINT m_nContrast;
	UINT m_nFilterValue;
	
	UINT m_nBitRate;
	UINT m_nCaptureMode;
	UINT m_nShowVolumeHelp;

    CString m_EncodePlayBackTarget; 
    CString m_EncodeType; 
	UINT m_nEncodeVideoEnable;
	UINT m_nEncodeVideoBitRate;
	UINT m_nEncodeSquarePixels;
	UINT m_nEncodeSofteningLevel;
	UINT m_nLargeImage;
	UINT m_nEncodeAudioEnable;
	UINT m_nEncodeAudioMode;
	UINT m_nEncodeAudioBitRate;
    UINT m_nCompressionEstimate;
	UINT m_nEncodeUseOptimumRate;
	UINT m_nEncodeStreamDataRate;
	UINT m_nEncodePackSize;
	UINT m_nEncodePacketsPerPack;
	UINT m_nStreamMixType;

	// New Properties added by Jeff Bryant (2/16/96)
	// Note: These are copies of system values
	int  m_nVolume;  // units of dB / 1000 (range -20,000 .. +20,000)
	int  m_nBalance; // units of dB / 1000 (-= attenuate left chan; += right)
	UINT m_nBlankLines; // Number of blank lines to throw away during capture

public:

//Implementation
                 
	UINT AutoDetect()				{ return m_nAutoDetect;	};
	UINT Ntsc()						{ return m_nNTSC; };
	UINT Composite()				{ return m_nComposite;};

	UINT AudioEnable()				{ return m_nAudioEnable;};
	UINT AudioMode()				{ return m_nAudioMode;};
	UINT AudioRate()		        { return m_nAudioRate;};
	
	UINT Hue()					    { return m_nHue;};
	UINT Saturation()				{ return m_nSaturation;};
	UINT Intensity()				{ return m_nIntensity;};
	UINT Contrast()					{ return m_nContrast;};
	UINT FilterValue()				{ return m_nFilterValue;};

	UINT BitRate()					{ return m_nBitRate;};
	UINT CaptureMode()				{ return m_nCaptureMode;};
	UINT ShowVolumeHelp()			{ return m_nShowVolumeHelp;};

	
    CString EncodePlayBackTarget()  { return m_EncodePlayBackTarget;};
    CString EncodeType()			{ return m_EncodeType;};

	UINT EncodeVideoEnable()		{ return m_nEncodeVideoEnable;};
	UINT EncodeVideoBitRate()		{ return m_nEncodeVideoBitRate;}
	UINT EncodeSquarePixels()		{ return m_nEncodeSquarePixels;};
	UINT EncodeSofteningLevel()		{ return m_nEncodeSofteningLevel;};
	UINT LargeImage()				{ return m_nLargeImage;};

	UINT EncodeAudioEnable()		{ return m_nEncodeAudioEnable;};
	UINT EncodeAudioMode()			{ return m_nEncodeAudioMode;};
	UINT EncodeAudioBitRate()		{ return m_nEncodeAudioBitRate;};

    UINT CompressionEstimate()      { return m_nCompressionEstimate;};

	UINT EncodeUseOptimumRate() 	{ return m_nEncodeUseOptimumRate;}; 
	UINT EncodeStreamDataRate() 	{ return m_nEncodeStreamDataRate;}; 
	UINT EncodePackSize()       	{ return m_nEncodePackSize;};       
	UINT EncodePacketsPerPack() 	{ return m_nEncodePacketsPerPack;}; 
	UINT StreamMixType()            { return m_nStreamMixType;};

	UINT EncodeVideoEnable(CString sTarget)		
	{ return GetSetting(sTarget,IDS_ENCODEVIDEOENABLE, DefaultEncodeVideoEnable());};

	UINT EncodeVideoBitRate(CString sTarget)	
	{ return GetSetting(sTarget,IDS_ENCODEVIDEORATE, DefaultEncodeVideoBitRate());}

	UINT EncodeSquarePixels(CString sTarget)	
	{ return GetSetting(sTarget,IDS_ENCODESQUAREPIXELS, DefaultEncodeSquarePixels());};

	UINT EncodeSofteningLevel(CString sTarget)	
	{ return GetSetting(sTarget,IDS_ENCODESOFTENINGLEVEL, DefaultEncodeSofteningLevel());};

	UINT LargeImage(CString sTarget)			
	{ return GetSetting(sTarget,IDS_LARGEIMAGE, DefaultLargeImage());};

	UINT EncodeAudioEnable(CString sTarget)		
	{ return GetSetting(sTarget,IDS_ENCODEAUDIOENABLE, DefaultEncodeAudioEnable());};

	UINT EncodeAudioBitRate(CString sTarget)		
	{ return GetSetting(sTarget,IDS_ENCODEAUDIORATE, DefaultEncodeAudioBitRate());};

	UINT EncodeAudioMode(CString sTarget)		
	{ return GetSetting(sTarget,IDS_ENCODEAUDIOMODE, DefaultEncodeAudioMode());};

// New Members added by Jeff Bryant (2/16/96)
	int  Volume()					{ return m_nVolume;};
	int  Balance()					{ return m_nBalance;};
	int  BlankLines()				{ return m_nBlankLines;};

	void SetAutoDetect(int nValue)	{ m_nAutoDetect = nValue;};
	void SetNTSC(int nValue)		{ m_nNTSC = nValue;};
	void SetComposite(int nValue)	{ m_nComposite = nValue;};

	void SetAudioEnable(int nValue)	{ m_nAudioEnable = nValue;};
	void SetAudioMode(int nValue)	{ m_nAudioMode = nValue;};
	void SetAudioRate(int nValue)	{ m_nAudioRate = nValue;};

	void SetHue(int nValue)			{ m_nHue = nValue;};
	void SetSaturation(int nValue)	{ m_nSaturation = nValue;};
	void SetIntensity(int nValue)	{ m_nIntensity = nValue;};
	void SetContrast(int nValue)	{ m_nContrast = nValue;};
	void SetFilterValue(int nValue)	{ m_nFilterValue = nValue;};
	
	void SetBitRate(int nValue)		{ m_nBitRate = nValue;};
	void SetCaptureMode(int nValue)	{ m_nCaptureMode = nValue;};
	void SetShowVolumeHelp(int nValue)	{ m_nShowVolumeHelp = nValue;};
	
    void SetEncodePlayBackTarget(CString sValue){ m_EncodePlayBackTarget = sValue;};
    void SetEncodeType(CString sValue){ m_EncodeType = sValue;};

	void SetEncodeVideoEnable(int nValue)		{ m_nEncodeVideoEnable = nValue;};
	void SetEncodeVideoBitRate(int nValue)		{ m_nEncodeVideoBitRate = nValue;}
	void SetEncodeSquarePixels(int nValue)		{ m_nEncodeSquarePixels = nValue;};
	void SetEncodeSofteningLevel(int nValue)	{ m_nEncodeSofteningLevel = nValue;};
	void SetLargeImage(int nValue)				{ m_nLargeImage = nValue;};

	void SetEncodeAudioEnable(int nValue)		{ m_nEncodeAudioEnable = nValue;};
	void SetEncodeAudioBitRate(int nValue)		{ m_nEncodeAudioBitRate = nValue;};
	void SetEncodeAudioMode(int nValue)			{ m_nEncodeAudioMode = nValue;};

    void SetCompressionEstimate (int nValue)    { m_nCompressionEstimate = nValue;};
	
	void SetEncodeUseOptimumRate(int nValue) 	{ m_nEncodeUseOptimumRate = nValue;}; 
	void SetEncodeStreamDataRate(int nValue) 	{ m_nEncodeStreamDataRate = nValue;}; 
	void SetEncodePackSize(int nValue)       	{ m_nEncodePackSize = nValue;};       
	void SetEncodePacketsPerPack(int nValue) 	{ m_nEncodePacketsPerPack = nValue;}; 
    void SetStreamMixType (int nValue)			{ m_nStreamMixType = nValue;};

	// New Members added by Jeff Bryant (2/16/96) 
    // TODO: call Bruce's Function on volume and balance
	void SetVolume(int nValue)                  { m_nVolume = nValue;};
	void SetBalance(int nValue)                 { m_nBalance = nValue;};
	void SetBlankLines(int nValue)              { m_nBlankLines = nValue;};

	UINT DefaultAutoDetect()			{ return( GetSetting( IDS_DEFAULTS, IDS_AUTODETECT) );};
	UINT DefaultNTSC()					{ return( GetSetting( IDS_DEFAULTS, IDS_NTSC)); };
	UINT DefaultComposite()				{ return( GetSetting( IDS_DEFAULTS, IDS_COMPOSITE));};

	UINT DefaultAudioEnable()			{ return( GetSetting( IDS_DEFAULTS, IDS_AUDIOENABLE));};
	UINT DefaultAudioMode()				{ return( GetSetting( IDS_DEFAULTS, IDS_AUDIOMODE));};
	UINT DefaultAudioRate()	            { return( GetSetting( IDS_DEFAULTS, IDS_AUDIORATE));};
	
	UINT DefaultHue()					{ return( GetSetting( IDS_DEFAULTS, IDS_HUE));};
	UINT DefaultSaturation()			{ return( GetSetting( IDS_DEFAULTS, IDS_SATURATION));};
	UINT DefaultIntensity()			    { return( GetSetting( IDS_DEFAULTS, IDS_INTENSITY));};
	UINT DefaultContrast()				{ return( GetSetting( IDS_DEFAULTS, IDS_CONTRAST));};
	UINT DefaultFilterValue()			{ return( GetSetting( IDS_DEFAULTS, IDS_FILTERVALUE));};
	
	UINT DefaultBitRate()				{ return( GetSetting( IDS_DEFAULTS, IDS_BITRATE));};
	UINT DefaultCaptureMode()			{ return( GetSetting( IDS_DEFAULTS, IDS_CAPTUREMODE));};
	UINT DefaultShowVolumeHelp()		{ return( GetSetting( IDS_DEFAULTS, IDS_SHOWVOLUMEHELP));};

    CString DefaultEncodePlayBackTarget() { return( GetStringSetting( IDS_DEFAULTS, IDS_ENCODEPLAYBACKTARGET));};
    CString DefaultEncodeType()			  { return( GetStringSetting( IDS_DEFAULTS, IDS_ENCODETYPE));};
	
	UINT DefaultEncodeVideoEnable()		{ return( GetSetting( IDS_DEFAULTS, IDS_ENCODEVIDEOENABLE));};
	UINT DefaultEncodeVideoBitRate()	{ return( GetSetting( IDS_DEFAULTS, IDS_ENCODEVIDEORATE));}
	UINT DefaultEncodeSquarePixels()	{ return( GetSetting( IDS_DEFAULTS, IDS_ENCODESQUAREPIXELS));};
	UINT DefaultEncodeSofteningLevel()	{ return( GetSetting( IDS_DEFAULTS, IDS_ENCODESOFTENINGLEVEL));};
	UINT DefaultLargeImage()			{ return( GetSetting( IDS_DEFAULTS, IDS_LARGEIMAGE));};

	UINT DefaultEncodeAudioEnable()		{ return( GetSetting( IDS_DEFAULTS, IDS_ENCODEAUDIOENABLE));};
	UINT DefaultEncodeAudioBitRate()	{ return( GetSetting( IDS_DEFAULTS, IDS_ENCODEAUDIORATE));};
	UINT DefaultEncodeAudioMode()		{ return( GetSetting( IDS_DEFAULTS, IDS_ENCODEAUDIOMODE));};
	
	UINT DefaultEncodeUseOptimumRate() 	{ return( GetSetting( IDS_DEFAULTS, IDS_ENCODEUSEOPTIMUMRATE));};
	UINT DefaultEncodeStreamDataRate() 	{ return( GetSetting( IDS_DEFAULTS, IDS_ENCODESTREAMDATARATE));};
	UINT DefaultEncodePackSize()       	{ return( GetSetting( IDS_DEFAULTS, IDS_ENCODEPACKSIZE));};
	UINT DefaultEncodePacketsPerPack() 	{ return( GetSetting( IDS_DEFAULTS, IDS_ENCODEPACKETSPERPACK));};
    UINT DefaultCompressionEstimate()   { return( GetSetting( IDS_DEFAULTS, IDS_COMPRESSIONESTIMATE));};
    UINT DefaultStreamMixType()			{ return( GetSetting( IDS_DEFAULTS, IDS_STREAMMIXTYPE));};

    UINT DefaultBlankLines()		    { return( GetSetting( IDS_DEFAULTS, IDS_BLANKLINES));};

	BOOL IsInitialized()	{ return m_State!=Uninitialized; }
    BOOL IsCapture()		{ return m_State==Capture; };
    BOOL IsEncode()			{ return m_State==Encode; };

    BOOL IsChangedAVI()		{ return m_bChangedAVI; };
    BOOL IsChangedMPG()		{ return m_bChangedMPG; };

	void SetModifiedAVI(BOOL bChanged = TRUE)	{ m_bChangedAVI = bChanged; };
	void SetModifiedMPG(BOOL bChanged = TRUE)	{ m_bChangedMPG = bChanged; };

private:

//data
    enum {Uninitialized, Capture, Encode} m_State;

    PVC_PROFILE_INFO m_pProf;
    
	VCUSER_HANDLE m_vh;
    DEVID m_DevID;
    BOOL m_bCapture;
    BOOL m_bOverlay;
    
	CString m_CString;
	CString m_iniFile;

	BOOL m_EnableHW;

	BOOL m_bChangedAVI;
	BOOL m_bChangedMPG;
	
public:

//Implementation

	
	void HW_SetHue(int nValue)		    { if (IsCapture() && m_EnableHW) VT_SetParam(m_DevID, HUE, nValue); };
	void HW_SetSaturation(int nValue)	{ if (IsCapture() && m_EnableHW) VT_SetParam(m_DevID, SATURATION, nValue); };
	void HW_SetIntensity(int nValue)	{ if (IsCapture() && m_EnableHW) VT_SetParam(m_DevID, INTENSITY, nValue); };
	void HW_SetContrast(int nValue)		{ if (IsCapture() && m_EnableHW) VT_SetParam(m_DevID, CONTRAST, nValue); };
	void HW_SetFilter(int nValue)		{ if (IsCapture() && m_EnableHW) VT_SetParam(m_DevID, FILTERVALUE, nValue); };
	void HW_SetBlankLines(int nValue)	{ if (IsCapture() && m_EnableHW) {VT_SetParam(m_DevID, BLANKLINES, nValue);BW_SetBlankLines(nValue);}};

    BOOL HW_SetVideoMode(UINT VidMode, UINT VidSource);
    BOOL HW_GetVideoMode(UINT *pVidMode, UINT *pVidSource);

	CString GetIniFile()  { return (m_iniFile);}

};
               

/////////////////////////////////////////////////////////////////////////////
// CBwdialogApp
// See bwdialog.cpp for the implementation of this class
//

class CBwdialogApp : public CWinApp
{
public:
	CBwdialogApp();
	~CBwdialogApp();
    virtual BOOL InitInstance( );
    CString m_StartupDirectory;
	LPCSTR 	m_pszOldHelpFilePath;


//Data
  CRegistrySettings m_Settings;

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CBwdialogApp)
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CBwdialogApp)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////
// Some  Usefull utilities
	int ComboStringToId(const int pIdTable[],int nIdTableSize,
		            char *pTestStr);
	void LoadComboBox(const int pIdTable[],int nIdTableSize,
				      CComboBox *pComboBox);


/////////////////////////////////////////////////////////////////////////////
