/*
 * bwdlgs.h
 *
 *
 * 32-bit Video Capture driver
 *
 * definition of interface between hardware-specific portions of
 * video tool driver and the configuration dialogs DLL
 * 
 * Mark Vrabel, March 96
 */

#ifndef __BWDLGS
#define __BWDLGS


/* 
 * defines which page is to be the default page 
 */
#define PAGE_SOURCE    0     
#define PAGE_DISPLAY   1         
#define PAGE_FORMAT    2      


/* information required to do minimum hardware
 * access to board.
 */
typedef struct _BW_BOARDINFO 
{
    VCUSER_HANDLE vh;
    DEVID DevID;
    BOOL bCapture;
    BOOL bOverlay;

} BW_BOARDINFO;

/* 
 * this structure is passed to the ConfigFormat ConfigSource and 
 * ConfigFormat functions. 
 */
typedef struct _BW_CAPTUREOPTIONS 
{
    unsigned long ulStructSize;
    LPVC_PROFILE_INFO lpProf;

    long nMode;
    long nIsBroadway;
	long nPrivateInterface;
    HWND hWndParent;

	/* user selectable settings */
	BW_CAPTURE_PREFERENCES *lpPreferences; //see bwuser.h

	    
} BW_CAPTUREOPTIONS;

/* 
 * this structure is passed to the ConfigRecode function. 
 */
typedef struct _BW_RECODEOPTIONS 
{
	/* dialog information */
    unsigned long ulStructSize; // size of this structure
    long nMode;					// dialog mode
    HWND hWndParent;			// parent window

	/* fixed settings taken from mpegtype.ini */
	long nEncodeUseOptimumRate;	// TRUE=use optimum rate; FALSE=use specified rate
	long nEncodeStreamDataRate;	// total stream data rate
	long nEncodePackSize;		// pack size
	long nEncodePacketsPerPack;	// packets per pack
	long nStreamMixType;		// stream mix type
	
	/* user selectable settings */
	BW_RECODE_PREFERENCES *lpPreferences; //see bwuser.h

} BW_RECODEOPTIONS;


#ifdef __cplusplus
extern "C" {
#endif


BOOL WINAPI BW_GetCaptureOptions(BW_BOARDINFO FAR *lpInfo, BW_CAPTUREOPTIONS FAR *lpOptions, BW_RECODEOPTIONS *pRealtime, int nFirstPage);
BOOL WINAPI BW_GetRecodeOptions(BW_RECODEOPTIONS FAR *lpOptions);
BOOL WINAPI BW_GetRealtimeRecodeOptions(BW_RECODEOPTIONS *pOptions);
BOOL WINAPI BW_GetMpegIniFileName (LPTSTR lpszName, UINT cbSize);

BOOL WINAPI BW_GetCaptureEncodeType (LPVC_PROFILE_INFO lpProf, LPTSTR lpszType, UINT cbSize);
BOOL WINAPI BW_SetCaptureEncodeType (LPVC_PROFILE_INFO lpProf, LPCTSTR lpszType);

int WINAPI BW_CapturePropertiesDialog (HWND hWnd, BW_PROPERTYOPTIONS *pOptions);
LPVOID WINAPI BW_CalProgressBarCreate (HWND hParent);
VOID WINAPI BW_CalProgressSetText (LPVOID lpPb, LPCTSTR lpstrText);
VOID WINAPI BW_CalProgressBarSetPercentage (LPVOID lpPb, UINT nPercent);
BOOL WINAPI BW_CalProgressBarContinue (LPVOID lpPb);
VOID WINAPI BW_CalProgressBarDestroy (LPVOID lpPb);

BOOL WINAPI BW_GetRecodeType (LPTSTR lpszType, UINT cbSize);
BOOL WINAPI BW_SetRecodeType (LPCTSTR lpszType);
int WINAPI BW_CalErrorDialog (HWND hParent);

#ifdef __cplusplus
}
#endif

#endif
