/*
 * overlay.cpp
 *
 * 32-bit Video Capture driver
 * exposed overlay API's
 *
 * Mark Vrabel, October 1995
 */
 
#define _INC_MSVIDEO

#include <windows.h>
#include <vfw.h>
#include <math.h>
#include <vcdriver.h>
#include <videobuf.h>
#include "hardware.h"
#include "overlay.h"


#ifndef FOURCC_YUY2
#define FOURCC_YUY2		mmioFOURCC('Y','U','Y','2')
#endif

/* convert a width in bits to a ULONG aligned width in bytes */
#define WIDTHBYTES(i)     ((unsigned)((i+31)&(~31))/8)


extern "C" void __stdcall SetConverterBlankLines (int nLines);

BOOL WINAPI 
VO_InitializeOverlay (LPHWINFO lpHw, VOVERLAY_HANDLE FAR *lpvoh)
{
    COverlay *pCOverlay;
    HRESULT hr;

    *lpvoh = 0;

    pCOverlay = new COverlay(lpHw, &hr);
    if (pCOverlay == NULL) {
        return FALSE;
    }

    if (FAILED(hr)) {
        delete pCOverlay;        
        return FALSE;
    }

    *lpvoh = (VOVERLAY_HANDLE)pCOverlay;
    return TRUE;
}


BOOL WINAPI 
VO_TerminateOverlay (VOVERLAY_HANDLE voh)
{
    COverlay *pCOverlay = (COverlay*)voh;

    if (pCOverlay != NULL) {
        delete pCOverlay;
    }

    return TRUE;
}


BOOL WINAPI 
VO_SetOverlaySourceFormat (VOVERLAY_HANDLE voh, LONG Width, LONG Height,
						   LONG BlankLines)
{
    COverlay *pCOverlay = (COverlay*)voh;

    if (pCOverlay == NULL) {
        return FALSE;
    }

    return pCOverlay->MediaSetSourceFormat(Width, Height,BlankLines);
}

BOOL WINAPI 
VO_SetOverlayBlankLines (VOVERLAY_HANDLE voh,LONG BlankLines)
{
    COverlay *pCOverlay = (COverlay*)voh;

	if (pCOverlay == NULL) {
        return FALSE;
    }

    return pCOverlay->MediaSetBlankLines(BlankLines);
}

BOOL WINAPI 
VO_SetOverlayDestFormat(VOVERLAY_HANDLE voh, LONG Width, LONG Height)
{
    COverlay *pCOverlay = (COverlay*)voh;

    if (pCOverlay == NULL) {
        return FALSE;
    }

    return pCOverlay->MediaSetDestFormat(Width, Height);
}


BOOL WINAPI 
VO_SetOverlayParentWindow(VOVERLAY_HANDLE voh, HWND hWnd)
{
    COverlay *pCOverlay = (COverlay*)voh;

    if (pCOverlay == NULL) {
        return FALSE;
    }

    return pCOverlay->MediaSetParentWindow((HWND)hWnd);
}


BOOL WINAPI 
VO_StartOverlay (VOVERLAY_HANDLE voh)
{
    COverlay *pCOverlay = (COverlay*)voh;

    if (pCOverlay == NULL) {
        return FALSE;
    }

    return pCOverlay->MediaPlay();
}


BOOL WINAPI 
VO_StopOverlay(VOVERLAY_HANDLE voh)
{
    COverlay *pCOverlay = (COverlay*)voh;

    if (pCOverlay == NULL) {
        return FALSE;
    }

    return pCOverlay->MediaStop();
}


BOOL WINAPI 
VO_PauseOverlay(VOVERLAY_HANDLE voh)
{
    COverlay *pCOverlay = (COverlay*)voh;

    if (pCOverlay == NULL) {
        return FALSE;
    }

    return pCOverlay->MediaPause();
}


BOOL WINAPI 
VO_ContinueOverlay(VOVERLAY_HANDLE voh)
{
    COverlay *pCOverlay = (COverlay*)voh;

    if (pCOverlay == NULL) {
        return FALSE;
    }

    return pCOverlay->MediaContinue();
}


BOOL WINAPI 
VO_DrawFrame(VOVERLAY_HANDLE voh)
{
    COverlay *pCOverlay = (COverlay*)voh;

    if (pCOverlay == NULL) {
        return FALSE;
    }

    return pCOverlay->MediaDraw();
}


BOOL WINAPI 
VO_DrawFrameNow (VOVERLAY_HANDLE voh)
{
    COverlay *pCOverlay = (COverlay*)voh;

    if (pCOverlay == NULL) {
        return FALSE;
    }

    return pCOverlay->MediaDraw();
}


BOOL WINAPI VO_NotifyStreamStart (VOVERLAY_HANDLE voh, BOOL bIsStreaming)
{
    COverlay *pCOverlay = (COverlay*)voh;

    if (pCOverlay == NULL) {
        return FALSE;
    }

    return pCOverlay->NotifyStreamStart (bIsStreaming);
}


COverlay::COverlay(LPHWINFO lpHw, HRESULT *phr)
{    
    m_lpHw		= lpHw;
    m_hWnd      = NULL;
    m_hDC       = NULL;
    m_hdd       = NULL;
    m_hThread   = NULL;
    m_ThreadId  = 0;
    m_DstWidth  = 0;
    m_DstHeight = 0;
    m_SrcWidth  = 0;
    m_SrcHeight = 0;
    m_Size      = 0;
    m_bDrawDib  = FALSE;    
    m_State     = Stopped;

    m_hCompleteEvent = NULL;
    m_hWorkEvent     = NULL;
    m_hBufferEvent   = NULL;
    m_bIsStreaming   = FALSE;

	m_BlankLines = 0;

  	m_biOut.biSize          = sizeof (BITMAPINFOHEADER);
    m_biOut.biPlanes        = 1;
    m_biOut.biXPelsPerMeter = 0;
    m_biOut.biYPelsPerMeter = 0;
    m_biOut.biClrUsed       = 0;
    m_biOut.biClrImportant  = 0;

    m_biOut.biCompression   = FOURCC_YUY2;
	m_biOut.biBitCount      = 16;
	
    m_hCompleteEvent = CreateEvent (NULL, FALSE, FALSE, NULL);
    m_hWorkEvent = CreateEvent (NULL, FALSE, FALSE, NULL);
    m_hBufferEvent = CreateEvent (NULL, FALSE, FALSE, NULL);

    if (m_hCompleteEvent == NULL || m_hWorkEvent == NULL || m_hBufferEvent  == NULL) {
        *phr = E_OUTOFMEMORY;
        return;
    }

	/* open and initialize a drawdib object */
    if ((m_hdd = DrawDibOpen()) == NULL) {
        *phr = E_OUTOFMEMORY;
        return;
    }

	*phr = NOERROR;
}


COverlay::~COverlay()
{
    MediaStop();
    DeleteContents();

    if (m_hCompleteEvent != NULL) {
        CloseHandle (m_hCompleteEvent);
    }
    if (m_hWorkEvent != NULL) {
        CloseHandle (m_hWorkEvent);
    }
    if (m_hBufferEvent  != NULL) {
        CloseHandle (m_hBufferEvent);
    }
    if (m_hdd != NULL) {
        DrawDibClose(m_hdd);
    }
}


void COverlay::DeleteContents( void )
{
    if (m_bDrawDib) {
        DrawDibEnd (m_hdd);
        m_bDrawDib = FALSE;
    }

    if (m_hWnd != NULL) {
		
		if (m_hDC != NULL) {
			ReleaseDC (m_hWnd, m_hDC);
			m_hDC = NULL;
		}
	}

    m_hWnd = NULL;
}


BOOL COverlay::MediaOpen(HWND hWnd) {

    m_hWnd = hWnd;
    m_hDC = GetDC(hWnd);

	/* initialize the bitmap header */
    m_biOut.biHeight    = m_SrcHeight;
    m_biOut.biWidth     = m_SrcWidth;
		
	m_biOut.biSizeImage = (DWORD)m_biOut.biHeight *
                          (DWORD)WIDTHBYTES(m_biOut.biWidth * m_biOut.biBitCount);

    if (!DrawDibBegin(m_hdd
                      ,m_hDC
                      ,m_DstWidth
                      ,m_DstHeight
                      ,&m_biOut
                      ,m_SrcWidth
                      ,m_SrcHeight
                      ,DDF_SAME_HDC|DDF_BUFFER
                      )) {


        return FALSE;
    } else {
        m_bDrawDib = TRUE;
    }

    return TRUE;
}


BOOL COverlay::MediaSetBlankLines(LONG BlankLines) {

    BOOL bOK = TRUE;

	MediaPause();

	// New line blanking capability
	m_BlankLines = BlankLines;

	MediaContinue();

	return bOK;

}


BOOL COverlay::MediaSetSourceFormat (LONG Width, LONG Height,
									 LONG BlankLines) 
{
    BOOL bOK = TRUE;

    /* 
     * pause while we update so we don't 
     * screw up the dib drawing!!! 
     */
	MediaPause();

    m_SrcWidth  = Width;
    m_SrcHeight = Height;
    m_Size      = ((m_SrcWidth*m_SrcHeight)*2);

    m_biOut.biWidth  = m_SrcWidth;
    m_biOut.biHeight = m_SrcHeight;

	// New line blanking capability
	m_BlankLines = BlankLines;

	m_biOut.biSizeImage = (DWORD)m_biOut.biHeight *
               (DWORD)WIDTHBYTES(m_biOut.biWidth * m_biOut.biBitCount);

	MediaContinue();

	return bOK;
}


BOOL COverlay::MediaSetDestFormat (LONG Width, LONG Height) 
{
    /* 
     * pause while we update so we don't 
     * screw up the dib drawing!!! 
     */
	MediaPause();

    m_DstWidth  = Width;
    m_DstHeight = Height;

	MediaContinue();

    return TRUE;
}


BOOL COverlay::MediaSetParentWindow(HWND hWnd) 
{
    if (CanStop() && hWnd != m_hWnd) {   

    	MediaPause();

        RECT Rect;
        if (!GetClientRect(hWnd, &Rect)) {
            return FALSE;
        }

        m_DstWidth  = Rect.right-Rect.left;
        m_DstHeight = Rect.bottom-Rect.top;
        
        DeleteContents();

        if (!MediaOpen(hWnd)) {
            DeleteContents();
            m_DstWidth  = 0;
            m_DstHeight = 0;
            return FALSE;
        }
				
	    LONG exstyle = ::GetWindowLong(m_hWnd, GWL_STYLE);
	    LONG style = ::GetWindowLong(m_hWnd, GWL_EXSTYLE);

      	MediaContinue();
    }
    
    return TRUE;
}


BOOL COverlay::MediaPlay(void) 
{
    if (!CanPlay()) {
        return FALSE;
	}
    
    /* verify the format has been set */
    if ((m_SrcWidth == 0) || (m_SrcHeight == 0)) {
        return FALSE;
    }

    /* create the thread */
    m_hThread = CreateThread(
		    NULL,
		    0,
		    (LPTHREAD_START_ROUTINE) ThreadProc,
		    this,
		    0,
		    &m_ThreadId);


    if (m_hThread == NULL) {
	    return(FALSE);
    }

    /* wait for completion of the init operation */
    WaitForSingleObject(m_hWorkEvent, INFINITE);

    /* check that the init succeeded */
    if ( (BOOL) m_FunctionResult == TRUE) {
        SetThreadPriority(m_hThread, m_bIsStreaming ?THREAD_PRIORITY_NORMAL:THREAD_PRIORITY_ABOVE_NORMAL);
		
        m_State = Playing;
	    return(TRUE);
    }

    /* init failed - wait for the thread to exit and clean up */
    WaitForSingleObject(m_hThread, INFINITE);

    CloseHandle(m_hThread);
    m_hThread = NULL;

    return FALSE;
}


BOOL COverlay::MediaStop( void ) 
{
    if (!IsInitialized()) {   
        return FALSE;
	}

    if (!CanStop()) {
        return TRUE;
    }
    
    SetEvent(m_hCompleteEvent);

    /* on this function, we don't wait for the completion event -
     * the thread just cleans up and exits. We wait for the thread to
     * terminate
     */
    WaitForSingleObject(m_hThread, INFINITE);
    CloseHandle(m_hThread);
    m_hThread = NULL;
    m_ThreadId = 0;
    
    m_State = Stopped;
    return TRUE;
}


BOOL COverlay::MediaPause( void ) 
{
    if (!CanPause())
        return TRUE;

    m_State = Paused;
    return TRUE;
}


BOOL COverlay::MediaContinue( void ) 
{
    if (!CanContinue())
        return TRUE;

    m_State = Playing;
    return TRUE;
}



BOOL COverlay::MediaDraw( void ) 
{
    if (!IsInitialized()) {   
        return FALSE;
	}

    if (!CanStop()) {
        return FALSE;
    }
	
    if (m_BufferState == Idle) {
        m_BufferState = Convert;
		SetEvent (m_hBufferEvent);
    }
    
    return TRUE;
}


BOOL COverlay::NotifyStreamStart(BOOL bIsStreaming) 
{
    if (!IsInitialized()) {   
        return FALSE;
	}

    m_bIsStreaming = bIsStreaming;
    if (CanStop()) {
        SetThreadPriority(m_hThread, m_bIsStreaming ?THREAD_PRIORITY_NORMAL:THREAD_PRIORITY_ABOVE_NORMAL);
    }

    return TRUE;
}


BOOL COverlay::MediaDrawFrameNow (void) 
{
	LPVIDEO_BUFFER lpvb = &m_lpHw->PassBufs.Bufs[m_lpHw->PassBufs.dwCurrentBuf];
	void *pInput = lpvb->lpData;

	SetConverterBlankLines (m_BlankLines);

	/* load Master1 DMA (passthru/overlay) */
	if (m_lpHw->PassBufs.dwNumberBufs > 1) 
	{
		lpvb = &m_lpHw->PassBufs.Bufs[m_lpHw->PassBufs.dwDmaIndex];

		VT_LoadDma(m_lpHw->DevID
			       ,MASTER1
		           ,lpvb->dwLength
			       ,lpvb->dwPhys
		           );
	}

	/* increment the buffer indexes */
	m_lpHw->PassBufs.dwDmaIndex = (m_lpHw->PassBufs.dwDmaIndex + 1) % m_lpHw->PassBufs.dwNumberBufs;
	m_lpHw->PassBufs.dwCurrentBuf = (m_lpHw->PassBufs.dwCurrentBuf + 1) % m_lpHw->PassBufs.dwNumberBufs;

	return TRUE;
}


/*
 * ThreadProc
 *
 * This will be called in response to worker thread creation.  
 * The arg parameter is a class pointer to a COverlay object.
 */
DWORD COverlay::ThreadProc(DWORD arg)
{
    COverlay *pCOverlay = (COverlay *) arg;
    
    return (pCOverlay->DoBufferProcessing ());
}


/* DoBufferProcessing
 * 
 * Handle proccesing and displaying of buffers within
 * the worker thread.
 */
DWORD COverlay::DoBufferProcessing( void )
{ 
    HANDLE haWaitObjects[2];
	DWORD dwWaitObject;
	void *pInput;

    haWaitObjects[0] = m_hCompleteEvent; 
    haWaitObjects[1] = m_hBufferEvent;

    m_BufferState = Idle;

    /* notify the caller we're done */
    m_FunctionResult = TRUE;
    SetEvent(m_hWorkEvent);
    
    for (;;) {

        /* wait for commands or buffers */
        dwWaitObject = WaitForMultipleObjects(2
			                                  ,haWaitObjects
											  ,FALSE
											  ,m_BufferState == Idle ?INFINITE:5);

        if (dwWaitObject == WAIT_OBJECT_0) {    // thread end request
            
			return (0);

        } else  {

			/* don't draw if paused... */
            if (m_State == Paused || m_BufferState == Idle) {
		        m_BufferState = Idle;
                continue;
            }

			/* can't draw if no window!!! */
            if (m_hWnd == NULL) {
		        m_BufferState = Idle;
                continue;
            }

			/* first time through do the YUV conversion */
			if (m_BufferState == Convert) {

				/* 
				 * conversion can be time consuming - check to see if there is 
				 * any chance we could stomp on a data transfer when we convert. 
				 * if there is go through the loop again and try later.
				 */
				LPVIDEO_BUFFER lpvb = &m_lpHw->CaptBufs.Bufs[m_lpHw->CaptBufs.dwCurrentBuf];

				/* check for errors */
				DWORD dwStart = m_lpHw->lpRegs->MasterStart2;
				DWORD dwPhys = lpvb->dwPhys;

				if (dwStart > dwPhys && dwStart < dwPhys + MAX_CAPT_SIZE*sizeof(DWORD))	{
					continue;			
				}	

	    		if (m_lpHw->lpRegs->Status & DONE_FLAG_1) {
					continue;			
				}	

				if (m_lpHw->lpRegs->Status & FIFO_OVERFLOW) {
					continue;			
				}	

    			if (!m_lpHw->bCanDrawNow) {
	                continue;
				}

				lpvb = &m_lpHw->PassBufs.Bufs[m_lpHw->PassBufs.dwCurrentBuf];
				pInput = lpvb->lpData;

				SetConverterBlankLines (m_BlankLines);
#ifdef _DEBUG_
				DWORD startTime = timeGetTime();
#endif 
				/* convert frame!!! */				  
				DrawDibDraw(m_hdd
    				,m_hDC     
	    		    ,0
					,0
			        ,m_DstWidth
				    ,m_DstHeight
					,&m_biOut
	                ,pInput
			        ,0
					,(m_SrcHeight == 240) ?0:1
					,(m_SrcWidth == 360) ?360:352
					,(m_SrcHeight == 240) ?m_SrcHeight:m_SrcHeight-1
		            ,DDF_SAME_HDC|DDF_DONTDRAW
		            );
#ifdef _DEBUG_
				static char buf[20];
				wsprintf (buf, "Conversion time = %ld\n", timeGetTime()-startTime);
				OutputDebugString(buf);
#endif
				SetConverterBlankLines (0);

		        /* load Master1 DMA (passthru/overlay) */
				if (m_lpHw->PassBufs.dwNumberBufs > 1) 
				{
					lpvb = &m_lpHw->PassBufs.Bufs[m_lpHw->PassBufs.dwDmaIndex];
					VT_LoadDma(m_lpHw->DevID
						       ,MASTER1
			                   ,lpvb->dwLength
						       ,lpvb->dwPhys
			                   );
				}

				/* increment the buffer indexes */
				m_lpHw->PassBufs.dwDmaIndex = (m_lpHw->PassBufs.dwDmaIndex + 1) % m_lpHw->PassBufs.dwNumberBufs;
				m_lpHw->PassBufs.dwCurrentBuf = (m_lpHw->PassBufs.dwCurrentBuf + 1) % m_lpHw->PassBufs.dwNumberBufs;

				m_BufferState = Draw;
			} 
			
			if (m_BufferState == Draw) {

				/* 
				 * second time through check to see if there is any chance we
				 * could stomp on a data transfer when we draw. if there is go 
				 * through the loop again and try later.
				 */
				LPVIDEO_BUFFER lpvb = &m_lpHw->CaptBufs.Bufs[m_lpHw->CaptBufs.dwCurrentBuf];

				/* check for errors */
				DWORD dwStart = m_lpHw->lpRegs->MasterStart2;
				DWORD dwPhys = lpvb->dwPhys;

				if (dwStart > dwPhys && dwStart < dwPhys + MAX_CAPT_SIZE*sizeof(DWORD))	{
					continue;			
				}	

	    		if (m_lpHw->lpRegs->Status & DONE_FLAG_1) {
					continue;			
				}	

				if (m_lpHw->lpRegs->Status & FIFO_OVERFLOW) {
					continue;			
				}	

    			if (!m_lpHw->bCanDrawNow) {
	                continue;
				}
#ifdef _DEBUG_
				DWORD startTime = timeGetTime();
#endif 
				/* display frame!!! */
				DrawDibDraw(m_hdd
    					    ,m_hDC     
    						,0
		    				,0
				            ,m_DstWidth
					        ,m_DstHeight
							,&m_biOut
				            ,pInput
    				        ,0
							,(m_SrcHeight == 240) ?0:1
							,(m_SrcWidth == 360) ?360:352
							,(m_SrcHeight == 240) ?m_SrcHeight:m_SrcHeight-1
							,DDF_SAME_HDC|DDF_UPDATE
			                );
#ifdef _DEBUG_
				static char buf[20];
				wsprintf (buf, "Draw time = %ld\n", timeGetTime()-startTime);
				OutputDebugString(buf);
#endif
				m_BufferState = Idle;				    
			}
		} 
	}

    return NOERROR;
}
