/*
 * overlay.h
 *
 * 32-bit Video Capture driver
 * overlay-specific data structures and definitions.
 *
 * Mark Vrabel, October 1995
 */

#ifndef _OVERLAY_H_
#define _OVERLAY_H_

class FrameConverter;

              
/*
 * Class definition for overlays
 */
class COverlay {
public:
    COverlay::COverlay(LPHWINFO lpHw, HRESULT *phr);
    COverlay::~COverlay();

    void COverlay::DeleteContents( void );

    BOOL COverlay::MediaOpen( HWND hWnd ); 
    BOOL COverlay::MediaSetSourceFormat(LONG Width, LONG Height,LONG BlankLines); 
	BOOL COverlay::MediaSetBlankLines(LONG BlankLines);
    BOOL COverlay::MediaSetDestFormat(LONG Width, LONG Height); 
    BOOL COverlay::MediaSetParentWindow(HWND hWnd); 

    BOOL COverlay::MediaPlay( void ); 
    BOOL COverlay::MediaStop( void ); 
    BOOL COverlay::MediaPause( void ); 
    BOOL COverlay::MediaContinue( void ); 

    BOOL MediaDraw( void ) ;
    BOOL NotifyStreamStart (BOOL bIsStreaming);

	BOOL MediaDrawFrameNow ( void ); 

    DWORD DoBufferProcessing( void );

private:    
    enum {Uninitialized, Stopped, Paused, Playing } m_State;
    enum {Idle, Convert, Draw } m_BufferState;

    LPHWINFO m_lpHw;

    HANDLE m_hCompleteEvent;
    HANDLE m_hWorkEvent;
    HANDLE m_hBufferEvent;
    HANDLE m_hThread;

    DWORD m_ThreadId;
    DWORD m_Size;

    BOOL m_bIsStreaming;
    BOOL m_FunctionResult;
    BOOL m_bDrawDib;
    
    HWND m_hWnd;
    HDC m_hDC;
    HDRAWDIB m_hdd;

    long m_DstWidth;
    long m_DstHeight;
    long m_SrcWidth;
    long m_SrcHeight;

	// Added by JFB to support blanklines
	int    m_BlankLines;  // # lines to blank (from bottom)
    
    BITMAPINFOHEADER m_biOut;

    BOOL CanPlay(){ return m_State==Stopped; };
    BOOL CanStop(){ return m_State==Playing || m_State==Paused; };
    BOOL CanPause(){return m_State==Playing; };
    BOOL CanContinue(){return m_State==Paused; };

    BOOL IsInitialized(){ return m_State!=Uninitialized; }

    static DWORD ThreadProc(DWORD arg);

};
 

#endif  // _OVERLAY_H_
